/****
 *
 * Exemplo: Uso Incorreto de free()
 *
 * Seo: D.10.2
 *
 ****/

#include <stdio.h>  /* Entrada e sada */
#include <stdlib.h> /* Funo exit()   */

/****
 *
 * TamanhoDeArquivo(): Calcula o tamanho de um arquivo
 *                     usando fseek() e ftell()
 *
 * Parmetros:
 *     stream (entrada) - stream associado ao arquivo cujo
 *                        tamanho ser calculado
 *
 * Retorno: -1, se ocorrer algum erro.
 *          O tamanho do arquivo, em caso contrrio.
 *
 * Observaes:
 *     1. O arquivo pode ser aberto em modo de texto ou
 *        binrio, mas deve permitir leitura.
 *     2. O stream deve permitir acesso direto
 *
 ****/
int TamanhoDeArquivo(FILE *stream)
{
      /* Tenta mover o indicador de posio */
      /* para o final do arquivo           */
   if (fseek(stream, 0, SEEK_END)) {
      return -1; /* O movimento no foi possvel */
   }

      /* Retorna o nmero de bytes do */
      /* incio at o final do arquivo  */
   return ftell(stream);
}

/****
 *
 * FechaArquivo(): Tenta fechar um arquivo e, quando isso
 *                 no  possvel, aborta o programa
 *
 * Parmetros:
 *      stream (entrada) - stream associado ao arquivo
 *      nomeArq (entrada) - nome do arquivo ou NULL
 *
 * Retorno: Nada
 *
 * Observao: Se o segundo parmetro no for NULL, o nome
 *             do programa aparece na mensagem de erro
 *
 ****/
void FechaArquivo(FILE *stream, const char *nomeArq)
{
   /***************************************************/
   /*                                                 */
   /* Se fclose() retornar um valor diferente de zero */
   /* ocorreu algum erro na tentativa de fechamento   */
   /* do arquivo. Neste caso, apresenta-se mensagem   */
   /* de erro e aborta-se o programa.                 */
   /*                                                 */
   /***************************************************/

   if (fclose(stream)) { /* Erro de fechamento */
      fprintf( stderr, "\a\n>>> Ocorreu erro no fechamento"
              " do arquivo %s.\n>>> O programa sera' "
              "encerrado.\n", nomeArq ? nomeArq : "");
      exit(1); /* Aborta o programa */
   }
}

/****
 *
 * main(): Determina o tamanho em bytes do arquivo cujo
 *         nome  introduzido via linha de comando
 *
 * Parmetros: argc (entrada) - Nmero de parmetros
 *                              de linha de comando
 *             argv (entrada) - Array de strings presentes
 *                              na linha de comando quando
 *                              o programa  executado
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(int argc, char *argv[])
{
   FILE  *stream;
   int    tamArquivo;

      /* Verifica se o programa foi invocado corretamente */
   if (argc != 2) { /* No foi */
      printf( "\n\t>>> Este programa deve ser usado assim:"
              "\n\t>>> %s nome-do-arquivo\n", argv[0] );
      return 1; /* Usurio no sabe usar o programa */
   }

      /* Tenta abrir o arquivo para leitura em modo */
      /* binrio e checa se o arquivo foi aberto    */
   if (!(stream = fopen(argv[1], "rb"))) {
      printf( "\n\t> Nao foi possivel abrir o arquivo %s\n",
              argv[1] );
      return 1; /* Erro de abertura de arquivo */
   }

      /* Calcula o tamanho do arquivo */
   tamArquivo = TamanhoDeArquivo(stream);

      /* Apresenta o resultado */
   if (tamArquivo < 0) {
      printf( "\n\t> Nao foi possivel determinar o tamanho "
              "de \"%s\" em modo binario", argv[1] );
   } else {
      printf( "\n\t> Tamanho de \"%s\" em modo binario: "
              "%d bytes\n", argv[1], tamArquivo );
   }

      /* Fecha o arquivo */
   FechaArquivo(stream, argv[1]);

      /* Tenta abrir o arquivo para leitura em modo */
      /* de texto e checa se o arquivo foi aberto   */
   if (!(stream = fopen(argv[1], "r"))) {
      printf( "\n\t> Nao foi possivel abrir o arquivo %s\n",
              argv[1] );
      return 1; /* Erro de abertura de arquivo */
   }

      /* Calcula o tamanho do arquivo */
   tamArquivo = TamanhoDeArquivo(stream);

      /* Apresenta o resultado */
   if (tamArquivo < 0) {
      printf( "\t> Nao foi possivel determinar o tamanho de"
              " \"%s\" em modo de texto", argv[1] );
   } else {
      printf( "\t> Tamanho de \"%s\" em modo de texto: "
              "%d bytes\n", argv[1], tamArquivo );
   }

      /* Aqui, no  preciso usar FechaArquivo(),  */
      /* pois o programa ser encerrado em seguida */
   fclose(stream);

   return 0;
}

/******************* Exemplos de Execuo ******************

C:\Programas> SecaoC_6_18 Tudor.bin

        > Tamanho de "Tudor.bin" em modo binario: 336 bytes
        > Tamanho de "Tudor.bin" em modo de texto: 336 bytes

C:\Programas> SecaoC_6_18 Tudor.txt

        > Tamanho de "Tudor.txt" em modo binario: 197 bytes
        > Tamanho de "Tudor.txt" em modo de texto: 197 bytes

[O programa funciona perfeitamente bem no sistema
Windows XP, mas o padro ISO de C no garante que
ele ir sempre funcionar.]

***********************************************************/
