/****
 *
 * Ttulo: Gerando Nmeros Aleatrios
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Gera um nmero aleatrio entre dois limites
 *            especificados pelo usurio
 *
 * Entrada: Valores inteiros que representam os limites
 *          do intervalo
 *
 * Sada: O nmero aleatrio gerado
 *
 ****/

#include <stdio.h>   /* printf()         */
#include <stdlib.h>  /* rand() e srand() */
#include <time.h>    /* time()           */
#include "leitura.h" /* LeInteiro()      */

/****
 *
 * NumeroAleatorio(): Retorna um nmero aleatrio entre os
 *                    valores recebidos
 *
 * Parmetros:
 *      m, n (entrada) - valores que definem o intervalo de
 *                       nmeros sorteados
 *
 * Retorno: O nmero aleatrio gerado
 *
 * Observaes:
 *      1.  indiferente se m <= n ou n <= m
 *      2. Quando chamada pela primeira vez, esta funo
 *         alimenta o gerador de nmeros aleatrios
 *
 ****/
int NumeroAleatorio(int m, int n)
{
   static int primeiraChamada = 1; /* Esta varivel checa se a  */
                                   /* funo est sendo chamada */
                                   /* pela primeira vez e deve  */
                                   /* ter durao fixa          */

      /* Se esta for a primeira chamada da funo, */
      /* alimenta o gerador de nmeros aleatrios  */
   if (primeiraChamada) {
         /* Alimenta o gerador de nmeros aleatrios */
      srand(time(NULL));

         /* A prxima chamada no ser mais a primeira */
      primeiraChamada = 0;
   }

      /* Leva em considerao o fato de */
      /* n poder ser menor do que m     */
   if (n < m) {
      return rand()%(m - n + 1) + n;
   }

      /* m  menor do que ou igual a n */
   return rand()%(n - m + 1) + m;
}

/****
 *
 * main(): L dois valores que representam limites de um
 *         intervalo, gera um nmero aleatrio entre os
 *         limites introduzidos e apresenta o resultado
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int i1, i2; /* Valores que definem o intervalo */

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa gera um numero aleatorio"
           "\n\t>>> entre os valores introduzidos.\n" );

   printf("\n\t>>> Primeiro limite do intervalo: ");
   i1 = LeInteiro();

   printf("\n\t>>> Segundo limite do intervalo: ");
   i2 = LeInteiro();

   printf( "\n\t>>> Numero aleatorio gerado: %d\n",
           NumeroAleatorio(i1, i2) );

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa gera um numero aleatorio
        >>> entre os valores introduzidos.

        >>> Primeiro limite do intervalo: 33

        >>> Segundo limite do intervalo: 55

        >>> Numero aleatorio gerado: 48

        >>> Obrigado por usar este programa.

**********************************************************/
