/****
 *
 * Ttulo: Calculadora Dirigida por Menu
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 23/07/2012
 * ltima modificao: 23/07/2012
 *
 * Descrio: Apresenta um menu no qual cada opo
 *            representa encerramento do programa ou
 *            uma operao aritmtica bsica, l a opo
 *            do usurio e efetua a respectiva operao
 *
 * Entrada: Caracteres isolados representando opes e
 *          nmeros de ponto flutuante representando os
 *          operandos das operaes aritmticas
 *
 * Sada: Resultados das operaes executadas
 *
 ****/

#include <stdio.h>   /* printf()     */
#include "leitura.h" /* LeituraFacil */

/****
 *
 * ApresentaMenu(): Apresenta um menu de opes
 *
 * Parmetros: nenhum
 *
 * Retorno: Nada
 *
 ****/
void ApresentaMenu(void)
{
   printf( "\n\n\t****** Opcoes ******\n"
           "\n\t[A]dicao"
           "\n\t[M]ultiplicacao"
           "\n\t[S]ubtracao"
           "\n\t[D]ivisao"
           "\n\t[E]ncerra o programa\n" );
}

/****
 *
 * main(): Apresenta um menu no qual cada opo representa
 *         encerramento do programa ou uma operao aritmtica
 *         bsica, l a opo e efetua a respectiva operao
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int    opcao;
   double x, y;

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa e' uma calculadora mambembe."
           "\n\t>>> Escolha a opcao 'E' para encerra-lo." );

      /* O lao encerra quando o usurio escolhe a opo 'E' */
   while (1) {

      ApresentaMenu();

         /* L a opo do usurio */
      printf("\n\t>>> Escolha sua opcao: ");
      opcao = LeOpcao("AMSDEamsde");

         /* Se a opo for 'E', encerra o lao */
      if (opcao == 'E' || opcao == 'e') {
         break; /* Encerra o lao while */
      }

      /*                                   */
      /* L os valores sobre os quais a    */
      /* operao aritmtica ser efetuada */
      /*                                   */

      printf("\n\t>>> Digite o primeiro valor: ");
      x = LeReal();

      printf("\n\t>>> Digite o segundo valor: ");
      y = LeReal();

         /* Efetua a operao escolhida */
      switch (opcao) {
         case 'A':
         case 'a': /* Adio */
            printf("\n\t> %f + %f = %f", x, y, x + y);
            break;
         case 'M':
         case 'm': /* Multiplicao */
            printf("\n\t> %f * %f = %f", x, y, x * y);
            break;
         case 'S':
         case 's': /* Subtrao */
            printf("\n\t> %f - %f = %f", x, y, x - y);
            break;
         case 'D':
         case 'd': /* Diviso */
            if (y) {
               printf("\n\t> %f / %f = %f", x, y, x / y);
            } else {
               printf("\n\t> O divisor nao pode ser zero");
            }
            break;
         default: /* No se deve chegar at aqui */
            printf("\nEste programa contem um erro!!!");
            break;
      } /* switch */
   } /* while */

      /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa e' uma calculadora mambembe.
        >>> Escolha a opcao 'E' para encerra-lo.

        ****** Opcoes ******

        [A]dicao
        [M]ultiplicacao
        [S]ubtracao
        [D]ivisao
        [E]ncerra o programa

        >>> Escolha sua opcao: a

        >>> Digite o primeiro valor: 2.5

        >>> Digite o segundo valor: 3.5

        > 2.500000 + 3.500000 = 6.000000

        ****** Opcoes ******

        [A]dicao
        [M]ultiplicacao
        [S]ubtracao
        [D]ivisao
        [E]ncerra o programa

        >>> Escolha sua opcao: d

        >>> Digite o primeiro valor: 5

        >>> Digite o segundo valor: 0

        > O divisor nao pode ser zero

        ****** Opcoes ******

        [A]dicao
        [M]ultiplicacao
        [S]ubtracao
        [D]ivisao
        [E]ncerra o programa

        >>> Escolha sua opcao: d

        >>> Digite o primeiro valor: 5

        >>> Digite o segundo valor: 2.5

        > 5.000000 / 2.500000 = 2.000000

        ****** Opcoes ******

        [A]dicao
        [M]ultiplicacao
        [S]ubtracao
        [D]ivisao
        [E]ncerra o programa

        >>> Escolha sua opcao: e

        >>> Obrigado por usar este programa.

**********************************************************/
