/****
 *
 * Ttulo: Conjectura de Collatz I
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/03/2013
 * ltima modificao: 16/03/2013
 *
 * Descrio: Encontra a sequncia de Collatz a partir
 *            de um dado nmero inteiro positivo
 *
 * Entrada: Valores inteiros positivos ou zero para
 *          encerrar o programa
 *
 * Sada: A sequncia de Collatz a partir de cada valor aceito
 *        pelo programa e nmero de termos de cada tal sequncia
 *
 * Definio: Seja n um inteiro positivo. Construa uma
 *            sequncia tendo n como primeiro termo e tal
 *            que cada termo seguinte  obtido dividindo-se
 *            o termo corrente por 2, se ele for par, ou
 *            multiplicando-se por 3 e somando-se 1, se ele
 *            for mpar. A Conjectura de Collatz prope que
 *            eventualmente um termo da sequncia assume o
 *            valor 1. Esta proposio nunca foi provada
 *            como verdadeira ou falsa (por isso 
 *            considerada uma conjectura).
 *
 ****/

#include <stdio.h>   /* printf() e putchar() */
#include "leitura.h" /* LeInteiro()          */

/****
 *
 * Collatz(): Apresenta na tela a sequncia de Collatz a
 *            partir de um dado nmero inteiro positivo e
 *            conta o nmero de termos dessa sequncia
 *
 * Parmetros:
 *      n (entrada) - termo inicial da sequncia
 *
 * Retorno: O nmero de termos da sequncia
 *
 ****/
int Collatz(int n)
{
   int cont = 1; /* Conta o nmero de termos da sequncia e */
                 /*  iniciado com 1 porque a sequncia tem */
                 /* pelo menos um termo, que  o parmetro  */

      /* O primeiro termo deve ser positivo */
   if (n <= 0) {
         /* Nada ser escrito, pois no existe sequncia */
      return 0;
   }

      /* Apresenta o primeiro termo na tela */
   printf("\n>>> %d%s", n, n > 1 ? " -> " : "\n");

      /* Determina e apresenta na tela os demais termos. */
      /* Se o lao a seguir no encerrar, i.e., se no   */
      /* for encontrado um termo igual a 1, voc ter    */
      /* encontrado um nmero que no satisfaz a         */
      /* conjectura de Collatz e ficar famoso.          */
   while (1) {
         /* Gera o prximo termo */
      if (!(n%2)) { /* O termo corrente  par */
         n = n/2; /* Calcula o prximo termo */
      } else { /* O termo corrente  mpar */
         n = 3*n + 1; /* Calcula o prximo termo */
      }

      printf("%d", n); /* Apresenta o novo termo */

      ++cont; /* Mais um termo foi gerado */

         /* Se o novo termo for 1, a sequncia est encerrada */
      if (n != 1) { /* Existe, pelo menos, mais um termo */
            /* Escreve seta que antecede o prximo termo */
         printf(" -> ");
      } else { /* No existem mais termos */
         putchar('\n'); /* Embelezamento */
         break; /* Encerra o lao */
      }
   }

   return cont;
}

/****
 *
 * main(): Encontra a sequncia de Collatz a partir de um dado
 *         nmero inteiro positivo
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int n, /* Nmero introduzido pelo usurio */
       nTermos;

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa encontra a sequencia de Collatz"
           "\n\t>>> a partir de um dado valor inteiro positivo."
           "\n\t>>> Digite zero para encerrar o programa.\n" );

      /* O lao encerra quando o usurio digitar 0 */
   while (1) {
         /* L o termo inicial da sequncia */
      printf( "\n\t>>> Digite um numero inteiro positivo"
              "\n\t>>> ou zero para encerrar o programa: ");
      n = LeInteiro();

      if (!n) { /* O usurio digitou zero */
         break; /* Encerra o lao */
      } else if (n < 0) { /* Valor digitado  negativo */
         printf("\a\n\t>>> Nao sao aceitos numeros negativos");
      } else {
            /* Apresenta a sequncia e obtm o nmero de termos */
         nTermos = Collatz(n);

            /* Apresenta o nmero de termos da sequncia */
         printf( ">>> A sequencia possui %d termos\n", nTermos );
      }
   }

      /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa encontra a sequencia de Collatz
        >>> a partir de um dado valor inteiro positivo.
        >>> Digite zero para encerrar o programa.

        >>> Digite um numero inteiro positivo
        >>> ou zero para encerrar o programa: 20

>>> 20 -> 10 -> 5 -> 16 -> 8 -> 4 -> 2 -> 1
>>> A sequencia possui 8 termos

        >>> Digite um numero inteiro positivo
        >>> ou zero para encerrar o programa: 0

        >>> Obrigado por usar este programa.

**********************************************************/
