/****
 *
 * Ttulo: Nmeros Primos III
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 02/06/2012
 * ltima modificao: 02/06/2012
 *
 * Descrio: Exibe todos os nmeros primos menores
 *            ou iguais a um determinado inteiro maior
 *            do que um
 *
 * Entrada: Valores inteiros positivos ou zero para
 *          encerrar o programa
 *
 * Sada: Os nmeros primos menores ou iguais a cada
 *        nmero introduzido ou uma mensagem informando
 *        que no h primos entre os dois nmeros.
 *
 ****/

#include <stdio.h>   /* printf()    */
#include <math.h>    /* sqrt()      */
#include "leitura.h" /* LeInteiro() */

/****
 *
 * LeNatural(): L um nmero natural (i.e., um nmero
 *              inteiro no negativo)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero natural lido
 *
 ****/
int LeNatural(void)
{
   int numNatural; /* O nmero a ser lido */

   numNatural = LeInteiro(); /* Primeira tentativa */

   while(numNatural < 0) {
      printf( "\a\n\t>>> Valor invalido. "
              "Tente novamente <<<\n\t> " );
      numNatural = LeInteiro(); /* Faz nova tentativa */
   }

   return numNatural;
}

/****
 *
 * EhPrimo2(): Verifica se um nmero inteiro
 *             maior do que um  primo ou no
 *
 * Parmetros:
 *      n (entrada): o nmero que ser testado
 *
 * Retorno: 1, se o nmero for primo
 *          0, se o nmero no for primo
 *         -1, se for indefinido (i.e., se n <= 1)
 *
 ****/
int EhPrimo2(int n)
{
   int i, raiz;

      /* O conceito de nmero primo no  definido */
      /* para nmeros inteiros menores do que dois */
   if (n <= 1) {
      return -1; /* Indefinido */
   }

   /*                                                */
   /* No mximo, um nmero  divisvel por sua raiz  */
   /* quadrada. Portanto, se no for encontrado um   */
   /* divisor para o parmetro no mximo igual a sua */
   /* raiz, a busca por um divisor  encerrada. Para */
   /* evitar que essa raiz seja recalculada a cada   */
   /* passagem no lao abaixo, armazena-se esse      */
   /* valor numa varivel local.                     */
   /*                                                */

      /* Calcula a raiz quadrada do nmero a ser testado */
   raiz = sqrt(n);

      /* Verifica se o nmero tem algum divisor. No  */
      /* mximo, um nmero  divisvel por sua raiz. */
   for (i = 2; i <= raiz; ++i) {
      if (!(n%i)) {
         return 0; /* Encontrado um divisor */
      }
   }

      /* No foi encontrado nenhum divisor para */
      /* o nmero dado. Portanto, ele  primo.  */
   return 1;
}

/****
 *
 * ExibePrimos(): Exibe todos os nmeros primos menores ou iguais
 *                a um determinado inteiro maior do que um
 *
 * Parmetros:
 *      n (entrada): o nmero que servir de referncia
 *
 * Retorno: Nada
 *
 ****/
void ExibePrimos(int n)
{
   int i;

      /* O valor do parmetro deve ser maior do que 1 */
   if (n <= 1) {
      printf("\nValor invalido\n");
      return;
   }

      /* O menor nmero primo  2 */
   printf("\n>>> Numeros primos entre 2 e %d:\n\t> ", n);

      /* Verifica quais so os primos */
   for (i = 2; i <= n; ++i) {
      if (EhPrimo2(i) > 0) {
         printf("%d\t", i); /* Encontrado um primo */
      }
   }

   printf("\n"); /* Embelezamento apenas */
}

/****
 *
 * main(): Exibe todos os nmeros primos menores ou
 *         iguais a um determinado inteiro maior do
 *         que um valor introduzido via teclado
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int num;

      /* Apresenta o programa e explica seu funcionamento */
   printf( "\n\t>>> Este programa verifica quais sao os numeros"
           "\n\t>>> primos entre dois e o valor digitado."
           "\n\t>>> Para encerra-lo, digite zero ou um.\n" );

      /* O lao principal do programa encerra */
      /* quando o usurio introduz 0 ou 1     */
   while (1) {
      printf("\n\t>>> Digite um numero inteiro que "
             "nao seja negativo:\n\t> ");
      num = LeNatural(); /* L o nmero */

      if (num <= 1) { /* Encerra o lao */
         break;
      }

         /* Encontra os nmeros primos     */
         /* entre 2 e o nmero introduzido */
      ExibePrimos(num);
   }

      /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo *******************

        >>> Este programa verifica quais sao os numeros
        >>> primos entre dois e o valor digitado.
        >>> Para encerra-lo, digite zero ou um.

        >>> Digite um numero inteiro que nao seja negativo:
        > 14

>>> Numeros primos entre 2 e 14:
        > 2     3       5       7       11      13

        >>> Digite um numero inteiro que nao seja negativo:
        > 1

        >>> Obrigado por usar este programa.

***********************************************************/
