/****
 *
 * Ttulo: Calculando MDC Usando o Algoritmo de Euclides
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/08/2012
 * ltima modificao: 16/08/2012
 *
 * Descrio: Calcula o mximo divisor comum de dois nmeros
 *
 * Entrada: Dois valores inteiros no negativos
 *
 * Sada: O MDC dos dois nmeros introduzidos
 *
 ****/

#include <stdio.h>   /* Entrada e sada */
#include "leitura.h" /* LeituraFacil    */

/****
 *
 * LeOpcaoSimNao(): L uma opo do tipo sim/no
 *
 * Parmetros: Nenhum
 *
 * Retorno: 1, se a opo for 's' ou 'S'
 *          0, se a opo for 'n' ou 'N'
 *
 ****/
int LeOpcaoSimNao(void)
{
   int op;

   op = LeOpcao("sSnN");

   if (op == 's' || op == 'S') {
      return 1;
   }

   return 0;
}

/****
 *
 * LeNatural(): L um nmero natural (i.e., um nmero
 *              inteiro no negativo)
 *
 * Parmetros: Nenhum
 *
 * Retorno: O nmero natural lido
 *
 ****/
int LeNatural(void)
{
   int numNatural; /* O nmero a ser lido */

inicio: /* Volta para c se o valor for invlido */
   numNatural = LeInteiro();

   while(numNatural < 0) {
     printf( "\a\n\t>>> Valor invalido. "
           "Tente novamente <<<\n\t> " );
     goto inicio; /* No faz mal a ningum */
   }

   return numNatural;
}

/****
 *
 * MDC2(): Calcula o MDC de dois nmeros naturais usando
 *         o algoritmo de Euclides
 *
 * Parmetros:
 *      x, y (entrada): nmeros cujo MDC ser calculado
 *
 * Retorno: O MDC dos dois nmeros recebidos como parmetros
 *
 ****/
int MDC2(int x, int y)
{
   int resto; /* Armazena o resto da diviso de x por y */

      /* O lao encerra quando o MDC for encontrado */
   while (1) {
         /* Calcula o resto da diviso de x por y */
      resto = x % y;

         /* Se a diviso no deixou resto, o MDC  y */
      if (!resto) {
         break;
      }

         /* Atualiza os valores de x e y para uma */
         /* nova tentativa de encontrar o MDC     */
      x = y;
      y = resto;
   }

   return y;
}

/****
 *
 * main(): L continuamente dois valores inteiros no negativos,
 *         calcula o MDC deles e apresenta o resultado
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int a, b; /* Dois valores que tero o MDC calculado */

      /* Apresenta o programa */
   printf("\n\t>>> Este programa o maximo divisor comum"
          "\n\t>>> de dois numeros inteiros positivos.\n");

      /* O lao encerra quando o usurio escolhe 'n' ou 'N' */
   do {
         /* L dois valores no negativos */
      printf("\n>>> Digite um numero inteiro que "
             "nao seja negativo:\n\t> ");
      a = LeNatural();

      printf("\n>>> Digite outro numero inteiro que "
             "nao seja negativo:\n\t> ");
      b = LeNatural();

         /* Se um dos dois valores for 0, informa   */
         /* que no h MDC. Caso contrrio, calcula */
         /* o MDC e o apresenta na tela.            */
      if (!a || !b) {
         printf("\n\t>>> Nao existe MDC de: %d e %d", a, b);
      } else {
         printf("\n\t>>> O MDC e': %d\n", MDC2(a, b));
      }

         /* Verifica se o usurio deseja continuar */
      printf("\n\t>>> Deseja continuar (s/n)? ");
   } while (LeOpcaoSimNao());

      /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo *******************

        >>> Este programa o maximo divisor comum
        >>> de dois numeros inteiros positivos.

>>> Digite um numero inteiro que nao seja negativo:
        > 7

>>> Digite outro numero inteiro que nao seja negativo:
        > 28

        >>> O MDC e': 7

        >>> Deseja continuar (s/n)? n

        >>> Obrigado por usar este programa.

***********************************************************/
