/****
 *
 * Ttulo: Nmeros de Fibonacci II
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 13/06/2012
 * ltima modificao: 13/06/2012
 *
 * Descrio: Verifica se cada nmero inteiro introduzido pelo
 *            usurio faz parte de uma sequncia de Fibonacci.
 *
 * Entrada: Valores inteiros positivos introduzidos pelo usurio
 *
 * Sada: Uma mensagem indicando se o nmero faz parte de uma
 *        sequncia Fibonacci ou no
 *
 * Observao: O programa encontra a maior sequncia de Fibonacci
 *             possvel para o tipo int e armazena-a num array.
 *
 ****/

#include <stdio.h>   /* Entrada e sada */
#include "leitura.h" /* LeituraFacil    */

   /* Tamanho mximo do array que armazenar */
   /* a sequncia de Fibonacci               */
#define MAX_ELEMENTOS 100

/****
 *
 * EmArray(): Verifica se um nmero inteiro encontra-se
 *            num array de inteiros
 *
 * Parmetros: ar[] (entrada): o array que ser pesquisado
 *             tam (entrada): nmero de elementos do array
 *             num (entrada): o nmero que ser procurado
 *
 * Retorno: ndice do nmero no array, se ele for encontrado
 *          -1, em caso contrrio
 *
 ****/
int EmArray(const int ar[], int tam, int num)
{
   int i;

      /* Compara cada elemento do array com o        */
      /* parmetro 'num'. Se for encontrado um       */
      /* elemento igual a 'num', retorna seu ndice. */
   for (i = 0; i < tam; ++i) {
      if (ar[i] == num) {
         return i; /* Encontrado um elemento igual */
      }
   }

      /* Valor recebido como parmetro */
      /* no foi encontrado no array   */
   return -1;
}

/****
 *
 * main(): Verifica se cada nmero inteiro introduzido pelo
 *         usurio faz parte de uma sequncia de Fibonacci
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int fib[MAX_ELEMENTOS] = {1, 1}, /* Armazenar a maior     */
                                    /* sequncia de Fibonacci */
       i, /* Indexador dos elementos da sequncia */
       nElem, /* Nmero de elementos armazenados em fib[] */
       nTeste; /* Nmero que ser testado */

      /* Encontra a maior sequncia de Fibonacci possvel */
      /* para o tipo int e armazena-a no array fib[]. O   */
      /* maior valor possvel  o ltimo encontrado antes */
      /* da ocorrncia de overflow.                       */
   for (i = 2; i < MAX_ELEMENTOS; ++i) {
      fib[i] = fib[i - 1] + fib[i - 2];

         /* Verifica se ocorreu overflow */
      if (fib[i] <= fib[i - 1] || fib[i] <= fib[i - 2]) {
         nElem = i; /* Ocorreu overflow. O maior nmero de   */
         break;     /* Fibonacci do tipo int foi encontrado. */
      }
   }

            /*****************************************
             * Aqui comea o programa para o usurio *
             *****************************************/

      /* Se o lao for foi encerrado porque o ndice do array */
      /* atingiu seu valor limite, talvez no tenha sido      */
      /* possvel armazenar a maior sequncia permitida. Logo,*/
      /* o programa precisa ser abortado voluntariamente para */
      /* que o tamanho mximo do array seja acrescido. Esse   */
      /* tipo de problema pode ser contornado por meio de     */
      /* alocao dinmica de memria (Captulo 11).          */
   if (i == MAX_ELEMENTOS) {
      printf( "\n\t>>> Este programa contem um erro. Entre"
              "\n\t>>> em contato com o programador.\n\n" );
      return 1;
   }

   printf( "\n\t>>> Este programa verifica se cada numero "
           "inteiro\n\t>>> introduzido faz parte de uma "
           "sequencia de Fibonacci. \n\t>>> Digite um "
           "valor menor do que 1 para encerrar.\n" );

   printf( "\n\t>>> Numero de elementos da maior "
           "sequencia de Fibonacci:\n\t>>> %d\n", nElem);

   printf( "\n\t>>> Maior numero de Fibonacci "
           "armazenado:\n\t>>> %d\n", fib[nElem-1]);

      /* A sada do lao ocorre quando o usurio   */
      /* digita um valor menor do que ou igual a 1 */
   while (1) {
      printf( "\n\nDigite o valor a ser testado:\n\t> ");
      nTeste = LeInteiro();

         /* Verifica se o valor digitado encerra o lao */
      if (nTeste < 1) {
         break;
      }

         /* Verifica se o valor digitado faz parte da  */
         /* sequncia de Fibonacci armazenada em fib[] */
      i = EmArray(fib, nElem, nTeste);

         /* Apresenta o resultado da busca no array */
      if (i >= 0) {
         printf( "\n\t>>> %d e' um numero de Fibonacci\n",
                 nTeste );

            /* Se o ndice retornado pelo array for 0,   */
            /* o valor introduzido pelo usurio foi 1, e */
            /* a menor sequncia de Fibonacci que contm */
            /* 1 possui 2 termos. Se o valor de i for    */
            /* diferente de 0, o nmero de termos da     */
            /* menor sequncia ser dada por i + 1.      */
         printf( "\n\t>>> A menor sequencia de Fibonacci"
                 "\n\t>>> que contem esse numero possui "
                 "%d elementos", !i ? 2 : i + 1 );
      } else {
         printf( "\n\t>>> %d NAO e' um numero de Fibonacci\n",
                 nTeste );
      }
   }

   printf("\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo *******************

        >>> Este programa verifica se cada numero inteiro
        >>> introduzido faz parte de uma sequencia de Fibonacci.
        >>> Digite um valor menor do que 1 para encerrar.

        >>> Numero de elementos da maior sequencia de Fibonacci:
        >>> 46

        >>> Maior numero de Fibonacci armazenado:
        >>> 1836311903


Digite o valor a ser testado:
        > 8

        >>> 8 e' um numero de Fibonacci

        >>> A menor sequencia de Fibonacci
        >>> que contem esse numero possui 6 elementos

Digite o valor a ser testado:
        > 0

        >>> Obrigado por usar este programa.

***********************************************************/

