/****
 *
 * Ttulo: Ordenao de Arrays pelo Mtodo da Bolha
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Implementao do mtodo de ordenao da bolha
 *
 * Entrada: Nada
 *
 * Sada: Um array antes e depois da ordenao
 *
 ****/

#include <stdio.h>

/****
 *
 * BubbleSort(): Ordena em ordem crescente um array de
 *               elementos do tipo int utilizando o
 *               mtodo de ordenao da bolha (Bubble Sort)
 *
 * Parmetros:
 *      ar (entrada e sada) - array que ser ordenado
 *      nElementos (entrada) - nmero de elementos do array
 *
 * Retorno: Nada
 *
 * Observao: Esse  o mtodo de ordenao mais simples
 *             de entender (e explicar), mas ele  um
 *             dos mais ineficientes.
 *
 ****/
void BubbleSort(int ar[], int nElementos)
{
   int i, aux,
       ordenado = 0; /* Informar se o array est ordenado. */
                     /*  importante que essa varivel seja */
                     /* iniciada com zero (v. abaixo).      */

   /************************************************************/
   /* Supe-se que inicialmente o array est ordenado fazendo- */
   /* se 'ordenado = 1'. Ento, se forem encontrados dois      */
   /* elementos fora de ordem, atribui-se novamente zero a     */
   /* 'ordenado', trocam-se os elementos de posio e          */
   /* recomea-se a verificao de ordem do array.             */
   /************************************************************/

      /* O lao encerra quando a varivel */
      /* 'ordenado' for diferente de 0    */
   while (!ordenado) {
         /* Supe que o array est ordenado */
      ordenado = 1;

      /*                                               */
      /* Compara cada elemento do array com o elemento */
      /* seguinte. Se for encontrado um elemento menor */
      /* do que seu antecessor, os dois elementos      */
      /* trocam de posio e o array  considerado     */
      /* fora de ordem (i.e., 'ordenado' assume zero). */
      /*                                               */

      for (i = 0; i < nElementos - 1; i++){
            /* Compara cada elemento com */
            /* o elemento seguinte       */
         if (ar[i] > ar[i+1]){
               /* Pelo menos dois elementos */
               /* esto fora de ordem       */
            ordenado = 0;

               /* Troca elementos adjacentes */
            aux = ar[i];
            ar[i] = ar[i+1];
            ar[i+1] = aux;
         }  /* if */
      } /* for */
   } /* while */
}

/****
 *
 * main(): Testa a funo BubbleSort()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int array[] = {12, 55, 21, 1, 6, 8, 17, 220, 5, 83},
       i, tamanho;

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa ordena um array de"
           "\n\t>>> inteiros usando o metodo da bolha.\n" );

      /* Calcula do tamanho do array */
   tamanho = sizeof(array)/sizeof(array[0]);

      /* Apresenta o array antes da ordenao */
   printf("\n\t>>> Array Original <<<\n\n\t> ");
   for (i=0; i < tamanho; i++) {
      printf("%d   ", array[i]);
   }

      /* Ordena o array */
   BubbleSort(array, tamanho);

      /* Apresenta o array depois da ordenao */
   printf("\n\n\t>>> Array Ordenado <<<\n\n\t> ");
   for (i=0; i < tamanho; i++) {
      printf("%d   ", array[i]);
   }

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa ordena um array de
        >>> inteiros usando o metodo da bolha.

        >>> Array Original <<<

        > 12   55   21   1   6   8   17   220   5   83

        >>> Array Ordenado <<<

        > 1   5   6   8   12   17   21   55   83   220

**********************************************************/
