/****
 *
 * Ttulo: Invertendo um Array
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 26/08/2012
 * ltima modificao: 26/08/2012
 *
 * Descrio: L um array de inteiros e inverte a
 *            ordem dos elementos
 *
 * Entrada: Valores inteiros que sero armazenados num
 *          array at que o nmero mximo de elementos
 *          seja atingido ou um valor que indica o
 *          encerramento da leitura seja lido
 *
 * Sada: O array contendo os elementos lidos invertido
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* printf()    */
#include "leitura.h" /* LeInteiro() */

/***************** Constantes Simblicas ******************/

#define MAX_ELEMENTOS   20 /* Nmero mximo de   */
                           /* elementos do array */
#define ENCERRA_LEITURA  0 /* Valor que encerra  */
                           /* a leitura de dados */

/************************* Aluses ************************/

extern int LeArray(int ar[], int max, int encerra);
extern void ExibeArrayInts(const int ar[], int tamanho);
extern void InverteArray(int ar[], int nElem);

/***************** Definies de Funes ******************/

/****
 *
 * LeArray(): L valores inteiros e armazena-os num array. A
 *            leitura encerra quando o usurio digitar o valor
 *            que especifica o encerramento da leitura ou
 *            quando o nmero mximo de elementos for atingido.
 *
 * Parmetros:
 *      ar (sada) - array que conter os elementos lidos
 *      max (entrada) - nmero mximo de elementos lidos
 *      encerra (entrada) - valor que encerra a leitura
 *
 * Retorno: O nmero de elementos lidos
 *
 ****/
int LeArray(int ar[], int max, int encerra)
{
   int nValoresLidos = 0, /* Nmero de valores lidos */
       elemento; /* Armazenar cada valor lido */

   printf("\a\n\t   *** Introduza os valores ***\n\n");

      /* O lao encerra quando o usurio digitar o valor que */
      /* especifica o encerramento da leitura ou quando o    */
      /* nmero mximo de elementos for atingido             */
   while (1) {
      if (nValoresLidos >= max) {/* Nmero mximo de      */
         break;                  /* elementos j foi lido */
      }

         /* L o prximo elemento do array */
      printf("\t>>> Valor: ");
      elemento = LeInteiro();

         /* Se o valor que identifica encerramento */
         /* de leitura foi lido, encerra o lao    */
      if (elemento == encerra) {
         break;
      }

         /* Armazena no array o elemento lido */
      ar[nValoresLidos] = elemento;

      ++nValoresLidos; /* Mas um elemento foi lido */
   }

   return nValoresLidos;
}

/****
 *
 * ExibeArrayInts(): Apresenta um array de ints na tela
 *
 * Parmetros:
 *      ar[] (entrada) - array que ser apresentado
 *      tamanho (entrada) - nmero de elementos do array
 *
 * Retorno: Nada.
 *
 ****/
void ExibeArrayInts(const int ar[], int tamanho)
{
   int i;

      /* Se o nmero de elementos for menor do que ou */
      /* igual a zero, escreve as chaves e retorna    */
   if (tamanho <= 0) {
      printf("\n\t{ }\n");
      return;
   }

      /* Escreve abre-chaves */
   printf("\n\t{ ");

      /* Escreve do primeiro ao penltimo elemento */
   for (i = 0; i < tamanho - 1; ++i) {
      printf("%d, ", ar[i]);
   }

      /* Escreve o ltimo elemento separadamente para */
      /* que no haja uma vrgula sobrando ao final   */
   printf("%d }\n",  ar[tamanho - 1]);
}

/****
 *
 * InverteArray(): Inverte a ordem dos elementos de um array
 *                 de ints
 *
 * Parmetros:
 *      ar (entrada) - array que ser invertido
 *      nElem (entrada) - nmero de elementos do array
 *
 * Retorno: Nada
 *
 ****/
void InverteArray(int ar[], int nElem)
{
   int aux, i;

      /* Troca as posies do primeiro e ltimo elementos, do */
      /* segundo e penltimo elementos, e assim por diante    */
   for (i = 0; i < nElem/2; ++i) {
      aux = ar[i];
      ar[i] = ar[nElem - i - 1];
      ar[nElem - i - 1] = aux;
   }
}

/****
 *
 * main(): L um array de inteiros e inverte a ordem dos
 *         seus elementos
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int array[MAX_ELEMENTOS], n;

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa le um conjunto de valores "
           "\n\t>>> inteiros e escreve-os em ordem inversa. "
           "\n\t>>> O valor %d encerra a leitura de dados e "
           "\n\t>>> o numero maximo de valores e' %d.\n",
           ENCERRA_LEITURA, MAX_ELEMENTOS );

      /* L os elementos do array e armazena */
      /* em 'n' o nmero de elementos lidos  */
   n = LeArray(array, MAX_ELEMENTOS, ENCERRA_LEITURA);

      /* Exibe o array em sua ordem original */
   printf("\n\t*** Ordem Original ***\n");
   ExibeArrayInts(array, n);

      /* Inverte a ordem dos elementos lidos */
   InverteArray(array, n);

      /* Exibe o array invertido */
   printf("\n\t*** Ordem Invertida ***\n");
   ExibeArrayInts(array, n);

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa le um conjunto de valores
        >>> inteiros e escreve-os em ordem inversa.
        >>> O valor 0 encerra a leitura de dados e
        >>> o numero maximo de valores e' 20.

           *** Introduza os valores ***

        >>> Valor: 1
        >>> Valor: 2
        >>> Valor: 3
        >>> Valor: 0

        *** Ordem Original ***

        { 1, 2, 3 }

        *** Ordem Invertida ***

        { 3, 2, 1 }

**********************************************************/
