/****
 *
 * Ttulo: Permutaes de Arrays
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 26/03/2013
 * ltima modificao: 26/03/2013
 *
 * Descrio: Apresenta na tela todas as permutaes
 *            de um array de elementos do tipo int
 *
 * Entrada: No h
 *
 * Sada: Todas as permutaes de um array de ints
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h> /* printf() */

/************************* Aluses ************************/

extern void BubbleSort(int ar[], int nElementos);
extern void ExibeArrayInts(const int ar[], int tamanho);
extern void TrocaElementos(int ar[], int i, int j);
extern int ProximaPermutacao(int ar[], int n);
extern void GeraPermutacoes(int ar[], int n);

/***************** Definies de Funes ******************/

/****
 *
 * BubbleSort(): Ordena em ordem crescente um array de
 *               elementos do tipo int utilizando o
 *               mtodo de ordenao da bolha (Bubble Sort)
 *
 * Parmetros:
 *      ar (entrada/sada) - array que ser ordenado
 *      nElementos (entrada) - nmero de elementos do array
 *
 * Retorno: Nada
 *
 * Observao: Este  o mtodo de ordenao mais simples
 *             de entender (e explicar), mas ele  um
 *             dos mais ineficientes.
 *
 ****/
void BubbleSort(int ar[], int nElementos)
{
   int i, aux,
       ordenado = 0; /* Informar se o array est   */
                     /* ordenado.  importante que  */
                     /* esta varivel seja iniciada */
                     /* com zero (v. abaixo).       */

   /******************************************************/
   /*                                                    */
   /* Supe-se que inicialmente o array est ordenado    */
   /* fazendo-se 'ordenado = 1'. Ento, se forem         */
   /* encontrados dois elementos fora de ordem, atribui- */
   /* se novamente zero a 'ordenado', trocam-se os       */
   /* elementos de posio e recomea-se a verificao   */
   /* de ordem do array.                                 */
   /*                                                    */
   /******************************************************/

      /* O lao encerra quando a varivel 'ordenado' */
      /* for avaliada como diferente de 0            */
   while (!ordenado) {
         /* Supe que o array est ordenado */
      ordenado = 1;

      /*************************************************/
      /*                                               */
      /* Compara cada elemento do array com o elemento */
      /* seguinte. Se for encontrado um elemento menor */
      /* do que seu antecessor, os dois elementos      */
      /* trocam de posio e o array  considerado     */
      /* fora de ordem (ordenado = 0).                 */
      /*                                               */
      /*************************************************/

      for (i = 0; i < nElementos - 1; i++){
            /* Compara cada elemento com */
            /* o elemento seguinte       */
         if (ar[i] > ar[i+1]){
               /* Pelo menos dois elementos */
               /* esto fora de ordem       */
            ordenado = 0;

               /* Troca elementos adjacentes */
            aux = ar[i];
            ar[i] = ar[i+1];
            ar[i+1] = aux;
         }  /* if */
      } /* for */
   } /* while */
}

/****
 *
 * ExibeArrayInts(): Apresenta um array de ints na tela
 *
 * Parmetros:
 *      ar[] (entrada) - array que ser apresentado
 *      tamanho (entrada) - nmero de elementos do array
 *
 * Retorno: Nada.
 *
 ****/
void ExibeArrayInts(const int ar[], int tamanho)
{
   int i;

      /* Se o nmero de elementos for menor do que ou */
      /* igual a zero, escreve as chaves e retorna    */
   if (tamanho <= 0) {
      printf("\n\t{ }\n");
      return;
   }

      /* Escreve abre-chaves */
   printf("\n\t{ ");

      /* Escreve do primeiro ao penltimo elemento */
   for (i = 0; i < tamanho - 1; ++i) {
      printf("%d, ", ar[i]);
   }

      /* Escreve o ltimo elemento separadamente para */
      /* que no haja uma vrgula sobrando ao final   */
   printf("%d }\n",  ar[tamanho - 1]);
}

/****
 *
 * TrocaElementos(): Troca as posies de dois elementos
 *                   de um array de elementos do tipo int
 *
 * Parmetros:
 *      ar[] (entrada/sada) - array contendo os elementos que
 *                             trocaro de posies
 *      i, j (entrada) - ndices dos elementos que sero trocados
 *
 * Retorno: Nenhum
 *
 ****/
void TrocaElementos(int ar[], int i, int j)
{
   int aux = ar[i]; /* Guarda o valor do elemento de ndice i */

      /* O valor do elemento ar[i] est guardado */
   ar[i] = ar[j];

      /* Atribui a ar[j] valor do elemento ar[i] */
   ar[j] = aux; /* O valor de ar[i] foi guardado em 'aux' */
}

/****
 *
 * ProximaPermutacao(): Gera a prxima permutao de um array de
 *                      elementos do tipo int usando ordenao
 *                      lexicogrfica
 *
 * Parmetros:
 *      ar (entrada/sada) - array cuja permutao ser gerada
 *      n (entrada) - nmero de elementos do array
 *
 * Retorno: 1, se uma permutao foi gerada;
 *          0, em caso contrrio
 *
 ****/
int ProximaPermutacao(int ar[], int n)
{
   int i, j;

      /*       >>> Passo 1 <<<       */
      /* Encontra o maior ndice i   */
      /* tal que: ar[i - 1] < ar[i]  */
   for (i = n - 1; i != 0 && ar[i - 1] >= ar[i]; i--) {
      ; /* Vazio */
   }

      /*                 >>> Passo 2 <<<                */
      /* Se o ndice i for igual a 0, no existe nenhum */
      /* elemento do array que seja maior do que seu    */
      /* antecessor. I.e., o array est em ordem        */
      /* decrescente e sua configurao atual constitui */
      /* a ltima permutao que pode ser gerada.       */
   if (!i) {
      return 0; /* No h mais permutao a ser gerada */
   }

      /*                 >>> Passo 3 <<<                 */
      /* Faz i assumir o maior o ndice tal que:         */
      /* ar[i] < ar[i + i]. Esse valor  exatamente i-1. */
   --i;

      /*            >>> Passo 4 <<<             */
      /* Encontra o maior ndice j tal que:     */
      /* ar[i] < ar[j]. Como ar[i] < ar[i + 1], */
      /* j deve existir.                        */
   for (j = n - 1; ar[j] <= ar[i]; j--) {
      ; /* Vazio */
   }

      /*    >>> Passo 5 <<<    */
      /* Troca ar[i] com ar[j] */
   TrocaElementos(ar, i, j);

      /*      >>> Passo 6 <<<       */
      /* Inverte os elementos do    */
      /* array de i + 1 at o final */
   for (i++, j = n - 1; j > i; j--, i++) {
      TrocaElementos(ar, i, j);
   }

   return 1; /* Uma permutao foi gerada */
}

/****
 *
 * GeraPermutacoes(): Gera e exibe na tela todas as permutaes
 *                    de um array de elementos do tipo int
 *                    usando ordenao lexicogrfica
 *
 * Parmetros:
 *      ar (entrada/sada): array cujas permutaes sero geradas
 *      n (entrada): nmero de elementos do array
 *
 * Retorno: Nenhum
 *
 ****/
void GeraPermutacoes(int ar[], int n)
{
         /*                                           */
         /* Para a tcnica de ordenao lexicogrfica */
         /* funcionar,  essencial que a gerao de   */
         /* permutaes seja iniciada com o array     */
         /* ordenado em ordem crescente.              */
         /*                                           */

      /* Ordena o array ar[] em ordem crescente */
   BubbleSort(ar, n);

   printf("\n     >>> Permutacoes <<<\n");

      /* Exibe todas as permutaes na tela. O lao termina */
      /* quando a funo ProximaPermutacao() retornar 0,    */
      /* indicando que no h mais permutao a ser gerada. */
   do {
      ExibeArrayInts(ar, n); /* Exibe a configurao */
                             /* corrente do array    */
   } while (ProximaPermutacao(ar, n));
}

/****
 *
 * main(): Apresenta na tela todas as permutaes de um array
 *         de elementos do tipo int
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int ar[] = {2, 1, 4, 3},
       tam;

      /* Apresenta o programa */
   printf("\n\t>>> Este programa exibe na tela todas as"
          "\n\t>>> permutacoes de um array de inteiros.\n");

      /* Calcula o tamanho do array ar[] */
   tam = sizeof(ar)/sizeof(ar[0]);

      /* Apresenta o array tal como ele foi iniciado */
   printf("\n    >>> Array original <<< ");
   ExibeArrayInts(ar, tam);

      /* Apresenta todas as permutaes do array */
   GeraPermutacoes(ar, tam);

   return 0;
}

/****************** Resultado do Programa ******************

        >>> Este programa exibe na tela todas as
        >>> permutacoes de um array de inteiros.

    >>> Array original <<<
        { 2, 1, 4, 3 }

     >>> Permutacoes <<<

        { 1, 2, 3, 4 }

        { 1, 2, 4, 3 }

        { 1, 3, 2, 4 }

        { 1, 3, 4, 2 }

        { 1, 4, 2, 3 }

        { 1, 4, 3, 2 }

        { 2, 1, 3, 4 }

        { 2, 1, 4, 3 }

        { 2, 3, 1, 4 }

        { 2, 3, 4, 1 }

        { 2, 4, 1, 3 }

        { 2, 4, 3, 1 }

        { 3, 1, 2, 4 }

        { 3, 1, 4, 2 }

        { 3, 2, 1, 4 }

        { 3, 2, 4, 1 }

        { 3, 4, 1, 2 }

        { 3, 4, 2, 1 }

        { 4, 1, 2, 3 }

        { 4, 1, 3, 2 }

        { 4, 2, 1, 3 }

        { 4, 2, 3, 1 }

        { 4, 3, 1, 2 }

        { 4, 3, 2, 1 }

***********************************************************/
