/****
 *
 * Ttulo: Leitura de Nomes
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 17/05/2012
 * ltima modificao: 18/05/2012
 *
 * Descrio: L um string e verifica se ele contm apenas letras
 *            e espaos em branco
 *
 * Entrada: Um string contendo apenas letras e espaos
 *
 * Sada: V. Exemplo de execuo
 *
 ****/

/************************ Includes ************************/

#include <stdio.h>   /* printf()   */
#include <ctype.h>   /* isalpha()  */
#include "leitura.h" /* LeString() */

/****************** Constantes Simblicas *****************/

#define MAX_NOME 20 /* Nmero mximo de caracteres num nome */

/***************** Definies de Funes ******************/

/****
 *
 * LeNome(): L um string via teclado e garante que ele s
 *           contm letras e espaos em branco
 *
 * Parmetros: nome (sada) - array que conter o string lido
 *             tam (entrada) - nmero de elementos do array
 *
 * Retorno: Nmero de caracteres digitados alm do limite permitido
 *
 * Observaes:
 *      1. Caracteres digitados alm do limite especificado
 *         sero ignorados
 *      2. O nico espao em branco permitido num nome  ' '
 *      3. Esta funo no testa se o string digitado 
 *         constitudo apenas por espaos em branco
 *
 ****/
int LeNome(char *nome, int tam)
{
   int   remanescentes; /* Nmero de caracteres digitados */
                        /* alm do permitido              */
   char *p; /* Usado para testar a validade do string */

   while (1) {
      printf("\nDigite o nome (max = %d letras):\n\t> ", tam-1);
      remanescentes = LeString(nome, tam);

         /* Verifica se o string lido  vazio */
      if (!*nome) {
         printf("\a\n\t>> Um nome nao pode ser vazio <<\n");
         continue; /* Salta o restante do lao */
      }

            /***************************************/
            /* Verifica se o string lido s contm */
            /* letras e espaos em branco          */
            /***************************************/

         /* O ponteiro p ser usado para testar o string */
      p = nome;

         /* O lao encerra quando p apontar para o caractere */
         /* terminal do string ou quando for encontrado um   */
         /* caractere que nem  letra nem espao em branco   */
      while (*p) {
            /* Se for encontrado um caractere que no  letra */
            /* ou espao em branco encerra este lao          */
         if( !isalpha(*p) && *p != ' ' ) {
            break;
         }

         p++; /* Aponta para o prximo caractere */
      }

         /* Se p apontar para o caractere terminal do */
         /* string, este foi aprovado no teste acima  */
      if (!*p) {
         break; /* String s contm letras e espaos */
      } else { /* String foi reprovado no teste */
         printf("\a\n\t>>> Um nome deve conter apenas <<<"
                "\n\t>>> letras e espacos em branco <<<\n");
      }
   }

   return remanescentes;
}

/****
 *
 * main(): Testa funo LeNome()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char umNome[MAX_NOME + 1];
   int  resto; /* Caracteres excedentes na leitura */

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa le nomes contendo apenas"
           "\n\t>>> letras e espacos em branco\n" );

      /* L o nome e atribui a 'resto' o nmero */
      /* de caracteres excedentes na leitura    */
   resto = LeNome(umNome, MAX_NOME + 1);

      /* Apresenta o nome que o programa efetivamente leu */
   printf("\n\t>>> O nome aceito foi: %s\n", umNome);

      /* Verifica se o usurio digitou caracteres demais */
   if (resto) {
      printf("\n\t>>> %d caracteres foram desprezados\n", resto);
   }

   return 0;
}

/******************* Exemplo de Execuo *******************

        >>> Este programa le nomes contendo apenas
        >>> letras e espacos em branco

Digite o nome (max = 20 letras):
        > Micro$oft

        >>> Um nome deve conter apenas <<<
        >>> letras e espacos em branco <<<

Digite o nome (max = 20 letras):
        >

        >> Um nome nao pode ser vazio <<

Digite o nome (max = 20 letras):
        > Oftalmotorrinolaringologista

        >>> O nome aceito foi: Oftalmotorrinolaring

        >>> 8 caracteres foram desprezados

***********************************************************/
