/****
 *
 * Ttulo: Leitura de Nmeros de Identificao
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: L um string de tamanho fixo contendo apenas dgitos
 *
 * Entrada: Strings digitados pelo usurio via teclado
 *
 * Sada: Mensagens informando se os strings so vlidos ou no
 *
 ****/

/************************ Includes ************************/

#include <stdio.h>   /* printf()   */
#include <string.h>  /* strlen()   */
#include <ctype.h>   /* isdigit()  */
#include "leitura.h" /* LeString() */

/****************** Constantes Simblicas *****************/

#define TAM_MATR  4 /* Nmero de caracteres numa matrcula */

/***************** Definies de Funes ******************/

/****
 *
 * LeIdentidade(): L um nmero de identificao via teclado
 *
 * Parmetros:
 *     id[] (sada) - array que conter o nmero lido
 *     tamArray (entrada) - nmero de elementos do array id[]
 *     prompt (entrada) - string que representa a poro inicial
 *                        do prompt a ser apresentado
 *
 * Retorno: O endereo do array apontado por id
 *
 * Observao: Assume-se que o tamanho do nmero de identificao
 *              igual ao tamanho do array id[] que a armazenar
 *             menos um.
 *
 ****/
char *LeIdentidade(char *id, int tamArray, const char *prompt)
{
   int   resto; /* Nmero de eventuais caracteres excedentes */
   char *p; /* Usado para testar o string */

      /* O lao encerra quando for lido um string vlido */
   while (1) {
         /* Apresenta prompt */
      printf( "\n%s com exatamente %d digitos:\n\t> ",
              prompt, tamArray - 1 );


      resto = LeString(id, tamArray); /* L entrada do usurio */

         /* Checa se o usurio digitou caracteres em excesso */
      if (resto) {
         printf("\a\n\t>>> Excesso de caracteres <<<\n");
         continue; /* Faz nova tentativa */
      }

         /* Verifica se foram digitados menos caracteres do que */
         /* esperado. Note o uso do operador de converso (int) */
         /* para converter o valor retornado por strlen(), que  */
         /*  do tipo size_t, e, assim, evitar comparao de um */
         /* valor sem sinal e outro com sinal.                  */
      if ( (int)strlen(id) != tamArray - 1 ) {
         printf("\a\n\t>>> Numero incompleto <<<\n");
         continue; /* Faz nova tentativa */
      }

         /**************************************************/
         /* Neste ponto, o tamanho do nmero est correto. */
         /* Resta testar se ele s contm dgitos.         */
         /**************************************************/

      p = id; /* O ponteiro p ser usado para testar o string */

         /* O lao encerra quando p apontar para o     */
         /* caractere terminal do string ou quando for */
         /* encontrado um caractere que no  dgito   */
      while (*p) {
            /* Se for encontrado um caractere que */
            /* no  dgito encerra este lao     */
         if(!isdigit(*p)) {
            break; /* Encerra o lao interno */
         }

         p++; /* Aponta para o prximo caractere */
      }

         /* Se p apontar para o caractere terminal do */
         /* string, este foi aprovado no teste acima  */
      if (!*p) {
         break; /* Nmero de identificao vlido */
      } else { /* Nmero de identificao invlido */
         printf("\a\n\t>>> \'%c\' nao e' digito <<<\n", *p);
      }
   }

   return id;
}

/****
 *
 * main(): L uma matrcula de quatro dgitos
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char matr[TAM_MATR + 1]; /* Array que armazenar a matrcula */

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa le uma matricula com 4 "
           "digitos.\n" );

      /* L um nmero de matrcula desprezando */
      /* o retorno da funo LeIdentidade()    */
   LeIdentidade(matr, TAM_MATR + 1, "Digite uma matricula");

      /* Apresenta a matrcula lida */
   printf("\n\t>>> Matricula digitada: %s\n", matr);

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa le uma matricula com 4 digitos.

Digite uma matricula com exatamente 4 digitos:
        > 12

        >>> Numero incompleto <<<

Digite uma matricula com exatamente 4 digitos:
        > 12ab

        >>> 'a' nao e' digito <<<

Digite uma matricula com exatamente 4 digitos:
        > 2132445

        >>> Excesso de caracteres <<<

Digite uma matricula com exatamente 4 digitos:
        > 4444

        >>> Matricula digitada: 4444

**********************************************************/
