/****
 *
 * Ttulo: Criptografia Chinfrim
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Criptografa e decifra frases criptografadas.
 *
 * Entrada: Frases introduzidas via teclado e opes
 *          associadas a criptografar/decifrar.
 *
 * Sada: As frases criptografadas ou decifradas, de acordo
 *        com a opo escolhida com pelo usurio.
 *
 ****/

#include <stdio.h>   /* printf()               */
#include <string.h>  /* strchr()               */
#include <ctype.h>   /* islower()              */
#include "leitura.h" /* LeString() e LeOpcao() */

   /* Tamanho do array que armazena os  */
   /* strings introduzidos pelo usurio */
#define TAMANHO_ARRAY 50

/****
 *
 * Criptografa(): Criptografa um string usando outro
 *                string como chave
 *
 * Parmetros:
 *      str (entrada/sada): string que ser criptografado
 *      chave (entrada): a chave usada na criptografia
 *
 * Retorno: O endereo do string criptografado
 *
 * Observao: Apenas letras minsculas so criptografadas
 *
 ****/
char *Criptografa(char *str, char const *chave)
{
   char *ptr = str; /* Guarda o endereo do string */

   /**************************************************/
   /* Substitui cada caractere no string que  letra */
   /* minscula pelo caractere que se encontra na    */
   /* chave na posio dada pela diferena entre o   */
   /* valor inteiro do caractere e o valor de 'a'.   */
   /**************************************************/

   while (*str) {
      if (islower(*str)) { /* Apenas letras minsculas */
                           /* so criptografadas       */

         /*                                                 */
         /* No array que contm a chave, o caractere que    */
         /* substituir 'a' encontra-se no ndice 0, o      */
         /* substituto de 'b' encontra-se no ndice 1, e    */
         /* assim por diante. Em geral, o substituto de um  */
         /* caractere c encontra-se no ndice dado por      */
         /* c - 'a'. Assim, o substituto do caractere       */
         /* apontado por 'str' no array chave[] encontra-se */
         /* no  ndice dado por: *str - 'a'.                */
         /*                                                 */

            /* Substitui o caractere apontado por 'str' pelo */
            /* caractere correspondente no array chave[]     */
         *str = chave[*str - 'a'];
      }

      str++; /* Passa para o prximo caractere */
   }

   return ptr; /* O endereo do string foi guardado em ptr */
}

/****
 *
 * Decifra(): Decifra um string usando outro string como chave
 *
 * Parmetros:
 *      str (entrada/sada) - string que ser decifrado
 *      chave (entrada) - a chave criptogrfica
 *
 * Retorno: O endereo do string decifrado
 *
 * Observao: Apenas letras minsculas so decifradas
 *
 ****/
char *Decifra(char *str, char const *chave)
{
   char *ptr = str; /* Guarda o endereo do string */

   /*********************************************************/
   /* Encontra o ndice no array chave[] de cada caractere  */
   /* que  letra no string criptografado,  soma o valor do */
   /* caractere que se encontra nesse ndice ao valor do    */
   /* caractere 'a' e substitui o caractere correspondente  */
   /* no string criptografado pelo resultado obtido.        */
   /*********************************************************/

   while (*str) {
      if (islower(*str)) { /* Apenas letras minsculas */
                           /* foram criptografadas     */

         /*                                                    */
         /* Encontra o ndice do caractere apontado por 'str'  */
         /* no array chave[]usando a expresso:                */
         /*         strchr(chave, *str) - chave                */
         /* Somando-se esse ndice a 'a', obtm-se o caractere */
         /* que substituir aquele apontado por 'str'.         */
         /*                                                    */

         *str = strchr(chave, *str) - chave + 'a';
      }

      str++; /* Passa para o prximo caractere */
   }

   return ptr; /* O endereo do string foi guardado em ptr */
}

/****
 *
 * main(): L frases via teclado e criptografa-as ou decifra-as
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   const char *chave = "tfhxqjemupidckvbaolrzwgnsy";
   char        str[TAMANHO_ARRAY];
   int         op;

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa criptografa e decifra frases"
           "\n\t>>> escritas com letras minusculas (apenas)." );

      /* O lao encerra quando o usurio digitar [ENTER] */
   while (1) {
      printf( "\n\nIntroduza uma frase ou apenas "
              "[ENTER] para encerrar:\n\t> " );
      LeString(str, TAMANHO_ARRAY);

         /* Verifica se o string  vazio */
      if (!*str) {
         break; /* Usurio digitou apenas [ENTER] */
      }

         /* L a opo do usurio */
      printf( "\nEscolha C para criptografar ou "
              "D para decifrar o texto:\n\t> " );
      op = LeOpcao("cCdD");

         /* Criptografa ou decifra, de acordo */
         /* com a opo do usurio            */
      if (op == 'c' || op == 'C') {
         printf( "\nTexto criptografado:\n\t> %s",
                 Criptografa(str, chave) );
      } else {
         printf( "\nTexto decifrado:\n\t> %s",
                 Decifra(str, chave) );
      } /* if */
   } /* while */

   printf( "\n>>> A chave criptografica usada foi:\n\t> %s."
           "\n>>> Por favor, memorize-a.\n", chave );

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa criptografa e decifra frases
        >>> escritas com letras minusculas (apenas).

Introduza uma frase ou apenas [ENTER] para encerrar:
        > O rato roeu a roupa do rei de Roma.

Escolha C para criptografar ou D para decifrar o texto:
        > c

Texto criptografado:
        > O otrv ovqz t ovzbt xv oqu xq Rvct.

Introduza uma frase ou apenas [ENTER] para encerrar:
        > O otrv ovqz t ovzbt xv oqu xq Rvct.

Escolha C para criptografar ou D para decifrar o texto:
        > d

Texto decifrado:
        > O rato roeu a roupa do rei de Roma.

Introduza uma frase ou apenas [ENTER] para encerrar:
        >

>>> A chave criptografica usada foi:
        > tfhxqjemupidckvbaolrzwgnsy.
>>> Por favor, memorize-a.

**********************************************************/
