/****
 *
 * Ttulo: Ocorrncias de Letras numa Palavra
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 26/08/2012
 * ltima modificao: 26/08/2012
 *
 * Descrio: Conta o nmero de ocorrncias de letras numa palavra
 *
 * Entrada: Strings
 *
 * Sada: Nmero de ocorrncia de cada letra de cada string
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* printf()               */
#include <ctype.h>   /* isalpha() e  toupper() */
#include "leitura.h" /* LeituraFacil           */

/***************** Constantes Simblicas ******************/

#define TAM_MAX_PALAVRA  30 /* Tamanho mximo de uma palavra */
#define NUMERO_DE_LETRAS 26 /* Numero de letras no alfabeto */

/***************** Definies de Funes ******************/

/****
 *
 * ContaLetrasEmString(): Conta o nmero de ocorrncias de cada
 *                        letra de um string
 *
 * Parmetros:
 *      str (entrada) - string cujas letras sero contadas
 *      contaLetras (entrada/sada) - array que contm a
 *                                    contagem de letras
 *
 * Retorno: Nada
 *
 ****/
void ContaLetrasEmString(const char* str, int contaLetras[])
{
   int indice; /* ndice no array de ocorrncias */
               /* que corresponde a uma letra    */

      /* Examina cada caractere do string apontado por 'str' e */
      /* conta as ocorrncias de cada letra encontrada        */
   while (*str) {
      if (isalpha(*str)) {
            /* Obtm o ndice no array de ocorrncias que       */
            /* corresponde  letra corrente. Esse ndice       */
            /* obtido convertendo-se a letra em maiscula e     */
            /* subtraindo-se seu valor de 'A'. Assim, 'A' ('a') */
            /* tem ndice 0, 'B' ('b') tem ndice 1, etc.       */
         indice = toupper(*str) - 'A';

            /* Incrementa o nmero de ocorrncias */
            /* da letra corrente                  */
         ++contaLetras[indice];
      }

      ++str; /* Passa para o prximo caractere */
   }
}

/****
 *
 * ApresentaFrequenciaDeLetras(): Apresenta o nmero de ocorrncias
 *                                de cada letra no array
 *
 * Parmetros:
 *      contaLetras[] (entrada) - array contendo o nmero de
 *                                ocorrncias de cada letra
 *                                numa palavra
 *
 * Retorno: Nada.
 *
 * Observao: Letras que no aparece na palavra no so levadas
 *             em considerao
 *
 ****/
void ApresentaFrequenciaDeLetras(const int contaLetras[])
{
   int ocorrencias, indice, c;

      /* Escreve cabealho da tabela */
   printf( "\nLetra\tOcorrencias"
           "\n=====\t===========\n\n" );

      /* Determina o nmero de ocorrncias de */
      /* cada letra no array contaLetras[]    */
   for (c = 'A'; c <= 'Z'; ++c) {

      /****************************************************/
      /* IMPORTANTE: Em qualquer cdigo de caracteres, as */
      /*             letras aparecem em ordem alfabtica  */
      /*             usual.  Mas, isso no significa que  */
      /*             qualquer caractere entre 'A' e 'Z'   */
      /*             ou entre 'a' e 'z'  letra. Por isso,*/
      /*             a chamada de isalpha()  necessria. */
      /****************************************************/

         /* Verifica se o caractere  uma letra */
      if (isalpha(c)) { /* Caractere  uma letra */
            /* Obtm o ndice no array de ocorrncias que    */
            /* corresponde  letra, convertendo-se a letra   */
            /* em maiscula e subtraindo-se seu valor de 'A' */
         indice = toupper(c) - 'A';

            /* Obtm o nmero de ocorrncias da letra */
         ocorrencias = contaLetras[indice];

            /* Apresenta apenas ocorrncias diferentes de zero */
         if (ocorrencias != 0) {
            printf("  %c\t %4d\n", c, ocorrencias);
         }
      }
   }
}

/****
 *
 * main(): Conta o nmero de ocorrncias de letras numa palavra
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int  i, contaLetras[NUMERO_DE_LETRAS];
   char palavra[TAM_MAX_PALAVRA + 1];

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa determina o numero de"
           "\n\t>>> ocorrencias de cada letra de uma palavra."
           "\n\t>>> Para encerra-lo digite apenas [ENTER].\n" );

      /* O lao encerra quando o usurio */
      /* digita apenas [ENTER]           */
   while (1) {
         /* L a palavra */
      printf( "\n\t>>> Digite a palavra (max = %d "
              "caracteres):\n\t\t> ", TAM_MAX_PALAVRA );
      LeString(palavra, TAM_MAX_PALAVRA + 1);

         /* Verifica se o usurio digitou apenas [ENTER] */
      if (!*palavra) { /* Digitou apenas [ENTER] */
         break; /* Encerra o lao */
      }

         /* Zera array que armazena as frequncias de letras */
      for (i = 0; i < NUMERO_DE_LETRAS; ++i) {
         contaLetras[i] = 0;
      }

         /* Efetua a contagem de frequncia de letras */
      ContaLetrasEmString(palavra, contaLetras);

         /* Apresenta o resultado */
      ApresentaFrequenciaDeLetras(contaLetras);
   }

      /* Despede-se do usurio */
   printf( "\n\t>>> Obrigado por usar este programa.\n");

   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Este programa determina o numero de
        >>> ocorrencias de cada letra de uma palavra.
        >>> Para encerra-lo digite apenas [ENTER].

        >>> Digite a palavra (max = 30 caracteres):
                > Anticonstitucionalissimamente

Letra   Ocorrencias
=====   ===========

  A         3
  C         2
  E         2
  I         5
  L         1
  M         2
  N         4
  O         2
  S         3
  T         4
  U         1

        >>> Digite a palavra (max = 30 caracteres):
                > [ENTER]

        >>> Obrigado por usar este programa.

**********************************************************/
