/****
 *
 * Ttulo: Aparando Strings
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Remove espaos em branco de strings,
 *            conforme considerados pela funo isspace()
 *
 * Entrada: No h
 *
 * Sada: Strings antes e depois da remoo de brancos
 *
 ****/

#include <stdio.h>  /* printf()  */
#include <string.h> /* strchr()  */
#include <ctype.h>  /* isspace() */

/****
 *
 * RemoveBrancosInicio(): Remove os espaos em branco [de acordo
 *                        com isspace()] no incio de um string
 *
 * Parmetros:
 *     str (entrada/sada) - string que ter seus espaos
 *                           em branco inicias removidos
 *
 * Retorno: Endereo do string modificado (i.e., sem
 *          espaos em branco iniciais)
 *
 ****/
char *RemoveBrancosInicio(char *str)
{
   char *p;
   int   i = 0;

      /* Se 'str' for NULL ou o string estiver   */
      /* vazio, esta funo no tem nada a fazer */
   if (!str || !*str) {
      return str; /* Nada a fazer */
   }

      /* Faz p apontar para o incio do string */
   p = str;

      /* Enquanto p no aponta para o caractere '\0' */
      /* e o caractere por ele apontado  espao,    */
      /* faz p apontar para o prximo caractere      */
   while ( p && isspace(*p) ) {
      ++p;
   }

      /* Se, neste ponto, p estiver apontando para o caractere */
      /* terminal do string, ele s continha espaos em branco */
   if (!*p) { /* O string s continha espaos */
      *str = '\0'; /* O string ficar vazio */
   } else { /* p aponta para o primeiro   */
            /* caractere que no  espao */

         /* Move cada caractere a partir daquele */
         /* apontado por p para o incio         */
      while (*p) {
         str[i] = *p;
         ++i;
         ++p;
      }

         /* O caractere terminal ainda    */
         /* no foi copiado para o string */
      str[i] = '\0';
   }

   return str; /* Retorna o endereo do string */
}

/****
 *
 * RemoveBrancosFim(): Remove os espaos em branco [de acordo
 *                     com isspace()] ao final de um string
 *
 * Parmetros:
 *     str (entrada/sada) - string que ter seus espaos em
 *                           branco finais removidos
 *
 * Retorno: Endereo do string sem espaos em branco finais
 *
 ****/
char *RemoveBrancosFim(char *str)
{
   char *p;

      /* Se 'str' for NULL ou o string estiver   */
      /* vazio, esta funo no tem nada a fazer */
   if (!str || !*str) {
      return str; /* Nada a fazer */
   }

      /* Faz p apontar para o caractere anterior */
      /* ao caractere terminal do string         */
   p = strchr(str, '\0') - 1;

      /* Enquanto p no aponta para o incio do string */
      /* e o caractere por ele apontado  espao, faz  */
      /* p apontar para o caractere anterior           */
   while ( p != str && isspace(*p) ) {
      --p;
   }

   /***********************************************************/
   /* Se p estiver apontando para o incio do string, ele s  */
   /* continha espaos em branco. Nesse caso, o string ficar */
   /* vazio (i.e., contendo apenas o caractere '\0'). Caso    */
   /* contrrio, p apontar para o ltimo caractere que no  */
   /* espao. Assim, coloca-se '\0' no prximo caractere      */
   /* apontado por p (que  um espao).                       */
   /***********************************************************/

   if (p == str) { /* String s continha espaos */
      *p = '\0'; /* Torna o string vazio */
   } else { /* p aponta para o ltimo     */
            /* caractere que no  espao */
      *++p = '\0'; /* Termina o string no prximo caractere */
   }

   return str; /* Retorna o endereo do string */
}

/****
 *
 * RemoveEspacos(): Remove os espaos em branco [de acordo com
 *                  isspace()] no incio e no final de um string
 *
 * Parmetros:
 *     str (entrada/sada) - string que ter seus espaos em
 *                           branco iniciais e finais removidos
 *
 ****/
char *RemoveEspacos(char *str)
{
   str = RemoveBrancosInicio(str); /* Remove espaos iniciais */

      /* Remove espaos finais e retorna o resultado */
   return RemoveBrancosFim(str);
}

/****
 *
 * main(): Testa as funes RemoveBrancosFim(),
 *         RemoveBrancosInicio() e RemoveEspacos()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char str1[] = "Espacos no final   \t\t";
   char str2[] = "   \t Espacos no inicio";
   char str3[] = "     \tEspacos no inicio e fim \t";
   char str4[] = "     \t   \t\t"; /* S espaos */
   char str5[] = "     \t   \t\t"; /* S espaos */

   printf("\n>>> String original:\n\t> \"%s\"", str1);
   printf( "\n>>> String sem espacos no final:\n\t>"
           " \"%s\"\n", RemoveBrancosFim(str1) );

   printf("\n>>> String original:\n\t> \"%s\"", str2);
   printf( "\n>>> String sem espacos no inicio:\n\t>"
           " \"%s\"\n", RemoveBrancosInicio(str2) );

   printf("\n>>> String original:\n\t> \"%s\"", str3);
   printf( "\n>>> String sem espacos no inicio ou final:"
           "\n\t> \"%s\"\n", RemoveEspacos(str3) );

   printf("\n>>> String original:\n\t> \"%s\"", str4);
   printf( "\n>>> String sem espacos no final:\n\t>"
           " \"%s\"\n", RemoveBrancosFim(str4) );

   printf("\n>>> String original:\n\t> \"%s\"", str5);
   printf( "\n>>> String sem espacos no inicio:\n\t>"
           " \"%s\"\n", RemoveBrancosInicio(str5) );

   printf("\n>>> String original:\n\t> \"%s\"", str5);
   printf( "\n>>> String sem espacos no inicio ou final:"
           "\n\t> \"%s\"\n", RemoveEspacos(str5) );

   return 0;
}

/****************** Resultado do Programa ******************

>>> String original:
        > "Espacos no final             "
>>> String sem espacos no final:
        > "Espacos no final"

>>> String original:
        > "      Espacos no inicio"
>>> String sem espacos no inicio:
        > "Espacos no inicio"

>>> String original:
        > "             Espacos no inicio e fim         "
>>> String sem espacos no inicio ou final:
        > "Espacos no inicio e fim"

>>> String original:
        > "                             "
>>> String sem espacos no final:
        > ""

>>> String original:
        > "                             "
>>> String sem espacos no inicio:
        > ""

>>> String original:
        > ""
>>> String sem espacos no inicio ou final:
        > ""

***********************************************************/
