/****
 *
 * Ttulo: Leitura de Datas com Validao II
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 22/06/2012
 * ltima modificao: 22/06/2012
 *
 * Descrio: L uma data e verifica sua validade
 *
 * Entrada: Valores inteiros introduzidos via teclado
 *          representando dia, mes e ano
 *
 * Sada: Uma mensagem indicando se a data  vlida ou no
 *
 ****/

/*********************** Includes *************************/

#include <stdio.h>   /* Entrada e sada             */
#include <string.h>  /* Processamento de strings    */
#include <ctype.h>   /* Classificao de caracteres */
#include "leitura.h" /* LeituraFacil                */

/***************** Constantes Simblicas ******************/

#define TAM_ARRAY_DATA          11
#define SECULO_ATUAL          2000
#define PRIMEIRO_ANO_BISSEXTO 1752

/************************* Aluses ************************/

extern void LeData(int *d, int *m, int *a);
extern int StrEmInt(const char *str, char **final);
extern int DigitosEmInt(char *digitos, const char *sep, int *num);
extern int ValidaData(char *data, int *d, int *m, int *a);
extern int EhAnoBissexto(int ano);
extern int EhDataValida(int dia, int mes, int ano);

/***************** Definies de Funes ******************/

/****
 *
 * LeData(): L uma data no formato dd/mm/aaaa ou dd/mm/aa
 *
 * Parmetros: d (sada) - o dia
 *             m (sada) - o mes
 *             a (sada) - o ano
 *
 * Retorno: Nada
 *
 * Observao: Se o ano s contiver dois dgitos, ele ser
 *             considerado no sculo atual
 *
 ****/
void LeData(int *d, int *m, int *a)
{
   char data[TAM_ARRAY_DATA]; /* Armazenar o string  */
                              /* contendo a data lida */
   int  resto; /* Caracteres eventualmente remanescentes */

inicio: /* Volta para c se a data for invlida */
   resto = LeString(data, TAM_ARRAY_DATA);

   while(resto || !ValidaData(data, d, m, a)) {
      printf( "\a\n\t>>> Data invalida. "
              "Tente novamente <<<\n\t> " );
      goto inicio; /* Sem espaguete */
   }
}

/****
 *
 * StrEmInt(): Converte um string em nmero inteiro (int)
 *             na base 10
 *
 * Parmetros:
 *      str (entrada) - string a ser convertido
 *     *final (sada) - se no for NULL, apontar para
 *                      o caractere no qual a converso
 *                       encerrada
 *
 * Retorno: O nmero resultante da converso, se ela for
 *          bem sucedida, ou zero, se ocorrer erro
 *
 * Observaes:
 *     1. Se houver erro de converso, o valor retornado
 *         ambguo, mas a ambiguidade pode ser resolvida
 *        por meio do segundo parmetro.
 *
 *     2. Espaamento entre um sinal e o primeiro dgito
 *        do string  interpretado como erro.
 *
 ****/
int StrEmInt(const char *str, char **final)
{
   int digito, /* Armazenar cada dgito do nmero */
       ehNegativo = 0, /* Indicar se o nmero  negativo */
       numero = 0; /* Armazenar o resultado da converso */
   char *p, *inicio;

      /* Guarda o incio do string */
   inicio = (char *) str; /* Converso explcita para */
                          /* agradar o compilador     */

      /*                            */
      /* Salta eventuais espaos em */
      /* branco no incio do string */
      /*                            */
   while (isspace(*str)) {
      str++;
   }

      /* Verifica se o string possui sinal */
   if (*str == '-') {
      ehNegativo = 1; /* Indica que o nmero  negativo */
      ++str; /* Passa para o prximo caractere */
   } else if (*str == '+') { /* Sinal positivo no    */
                             /* levado em considerao */
      ++str; /* Passa para o prximo caractere */
   }

      /* Neste ponto foram saltados eventuais espaos */
      /* em branco iniciais e sinal. Agora, deve-se   */
      /* verificar se o string ficou vazio.           */
   if (!*str) {
         /* Se o ponteiro 'final' no for NULL, faz   */
         /* ele apontar para o incio do string.      */
      if (final) {
         *final = inicio;
      }

      return 0; /* No houve converso alguma */
   }

      /*                            */
      /* Salta eventuais espaos em */
      /* branco ao final do string  */
      /*                            */

      /* Faz p apontar para o ltimo caractere do string */
   p = strchr(str, '\0') - 1;

      /* Salta espaos em branco ao final do string */
   while (isspace(*p)) {
      p--;
   }

      /* Neste ponto, p aponta para o ltimo caractere do */
      /* string que no  espao em branco. Faz-se este   */
      /* ponteiro apontar para o caractere seguinte para  */
      /* que ele se torne delimitador dos caracteres do   */
      /* string que sero convertidos.                    */
   ++p;

      /* O lao encerra quando o string delimitado */
      /* por p for totalmente explorado ou se      */
      /* ocorrer algum erro durante a converso    */
   while (str != p) {
      if (isdigit(*str)) { /* Mais um dgito encontrado */
            /* Obtm o nmero equivalente ao dgito */
         digito = *str - '0'; /* Vlido para qualquer */
                              /* cdigo de caracteres */

            /* Atualiza o valor numrico (10  a base) */
         numero = numero*10 + digito;
      } else { /* Caractere esprio encontrado */
            /* Se o parmetro 'final' no for NULL, faz */
            /* o ponteiro apontar para o caractere que  */
            /* causou o final da converso              */
         if (final) {
            *final = (char *) str; /* Se (char *) no for */
                        /* usado o compilador apresentar */
                        /* mensagem de advertncia        */
         }

            /* O resultado obtido at aqui ser retornado,*/
            /* mas a converso pode ter sido mal sucedida */
         return numero;
      }

         /* O nmero sendo convertido  positivo. */
         /* Portanto, se ele assumir um valor     */
         /* negativo, ocorreu overflow.           */
      if (numero < 0) {
         printf("\a\n\t>>> Overflow!!! <<<\n");

            /* Se o parmetro 'final' no for NULL, faz */
            /* o ponteiro apontar para o caractere que  */
            /* causou o final da converso              */
         if (final) {
            *final = (char *) str; /* (char *) agrada */
                                   /* o compilador    */
         }

         return 0; /* A converso foi mal sucedida */
      }

      str++; /* Passa para o prximo caractere */
   }

      /* Se o nmero for negativo, efetua a correo */
   if (ehNegativo) {
      numero = -numero; /* O nmero  negativo */
   }

      /* Se o parmetro 'final' no for NULL, faz o */
      /* ponteiro apontar para o caractere terminal */
   if (final) {
      *final = strchr(str, '\0');
   }

   return numero; /* A converso foi bem sucedida */
}

/****
 *
 * DigitosEmInt(): Extrai um token de um string e converte-o
 *                 em nmero inteiro (int) na base 10
 *
 * Parmetros:
 *      *digitos (entrada/sada) - string cujo token ser
 *                                 extrado e convertido
 *      *sep (entrada) - string contendo os separadores de
 *                       tokens
 *      *num (sada) - nmero resultante da converso
 *
 * Retorno: 0, se a operao for bem sucedida;
 *          1, em caso contrrio
 *
 * Observao: Se houver mais de um token a ser processado num
 *             mesmo string, a primeira chamada desta funo
 *             deve usar o endereo do string como primeiro
 *             parmetro. Em chamadas subsequentes, esse
 *             parmetro deve ser NULL.
 *
 ****/
int DigitosEmInt(char *digitos, const char *sep, int *num)
{
   char *token, /* Armazena o token extrado do string */
        *erro; /* Indicar se sobrou algum caractere na */
               /* converso de um token para inteiro    */

   token = strtok(digitos, sep); /* Extrai o token */

      /* Se o token no pode ser obtido, o */
      /* nmero tambm no pode ser obtido */
   if (!token) {
      return 1; /* Impossvel obter nmero */
   }

      /* Tenta converter o token  em inteiro */
   *num = StrEmInt(token, &erro);

      /* Se algum caractere deixou de ser */
      /* convertido, o nmero  invlido  */
   if (*erro) {
      return 1; /* Sobrou algo na converso */
   }

   return 0;
}

/****
 *
 * ValidaData(): Separa uma data contida num string em dia,
 *               ms e ano, e tenta valid-la
 *
 * Parmetros:
 *     data (entrada/sada) - string contendo a data a ser
 *                            validada
 *     d (sada) - o dia
 *     m (sada) - o ms
 *     a (sada) - o ano
 *
 * Retorno: 1, se a data for vlida
 *          0, se a data no for vlida
 *
 * Observao:
 *     1. Se o ano contiver apenas dois dgitos, ele ser
 *        considerado no sculo atual.
 *     2. O string (primeiro parmetro)  modificado por
 *        esta funo.
 *
 ****/
int ValidaData(char *data, int *d, int *m, int *a)
{
      /* Tenta obter o dia */
   if (DigitosEmInt(data, "/", d)) {
         /* O dia no pode ser obtido */
      return 0; /* Dia invlido */
   }

      /* Tenta obter o ms */
   if (DigitosEmInt(NULL, "/", m)) {
         /* O ms no pode ser obtido */
      return 0; /* Ms invlido */
   }

      /* Tenta obter o ano */
   if (DigitosEmInt(NULL, "/", a)) {
         /* O ano no pode ser obtido */
      return 0; /* Ano invlido */
   }

      /* Se o ano tem menos de quatro dgitos,  */
      /* assume-se que  um ano do atual sculo */
   if (*a < 100) {
      *a = *a + SECULO_ATUAL;
   }

      /* A funo EhDataValida() completa o servio */
   return EhDataValida(*d, *m, *a);
}

/****
 *
 * EhAnoBissexto(): Verifica se um ano  bissexto
 *
 * Parmetros: ano (entrada): o ano que ser testado
 *
 * Retorno: 1, se o ano for bissexto
 *          0, se o ano no for bissexto
 *
 * Observao: Um ano  bissexto quando:
 *             * Ele  mltiplo de 400 ou
 *             * Ele  mltiplo de 4, mas no
 *                mltiplo de 100
 *
 ****/
int EhAnoBissexto(int ano)
{
      /* Se o ano for anterior ao primeiro */
      /* ano considerado bissexto, ele no */
      /* pode ser assim considerado        */
   if (ano < PRIMEIRO_ANO_BISSEXTO) {
      return 0;
   }

   return !(ano%400) || (!(ano%4) && ano%100);
}

/****
 *
 * EhDataValida(): verifica se uma data  vlida
 *
 * Parmetros: dia (entrada) - o dia
 *             mes (entrada) - o ms
 *             ano (entrada) - o ano
 *
 * Retorno: 1, se a data for vlida; 0, em caso contrrio
 *
 ****/
int EhDataValida(int dia, int mes, int ano)
{
      /* No existe dia menor do que ou igual */
      /* a zero, nem dia maior do que 31      */
   if ( dia <= 0 || dia > 31) {
      return 0; /* Dia invlido */
   }

      /* Verifica se o dia  vlido, o que depende do ms */
   switch(mes) {
      case 1: /* Estes meses sempre tm 31 dias */
      case 3:
      case 5:
      case 7:
      case 8:
      case 10:
      case 12:
         return 1;
      case 4: /* Estes meses sempre tm 30 dias */
      case 6:
      case 9:
      case 11:
            /* Se o ms tiver mais de 30 */
            /* dias, ele  invlido      */
         if (dia > 30) {
            return 0;
         } else {
            return 1;
         }
      case 2: /* Este ms pode ter 28 ou 29 dias */
            /* Se o ms tiver mais de 29 */
            /* dias, ele  invlido      */
         if (dia > 29) {
            return 0;
         } else if (dia < 29) {
            return 1; /* Ms tem 28 dias */
         } else if (EhAnoBissexto(ano)) {
            return 1; /* Ms tem 29 dias */
         } else {
            return 0;
         }
      default: /* A execuo no deve chegar at aqui */
         printf("\nOcorreu um erro sobrenatural");
   }

      /* A execuo tambm no se deve chegar at aqui */
   printf("\nOcorreu outro erro sobrenatural");

   return 0; /* Esta instruo nunca deve ser executada */
}

/****
 *
 * main(): L uma data e verifica sua validade
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int dia, mes, ano;

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa verifica a validade de uma data"
           "\n\t>>> introduzida no formato dd/mm/aaaa ou "
           "dd/mm/aa.\n\t>>> Se o ano tiver apenas dois digitos,"
           " sera' \n\t>>> assumido o seculo atual.\n" );

   printf( "\n\t>>> Digite a data no formato dd/mm/aaaa "
           "ou dd/mm/aa:\n\t> " );
   LeData(&dia, &mes, &ano);

   printf( "\n\t>>> A data introduzida foi: \n\t> "
           "%.2d/%.2d/%.2d\n", dia, mes, ano );

   return 0;
}

/******************* Exemplos de Execuo ******************

 Exemplo 1:
 =========

        >>> Este programa verifica a validade de uma data
        >>> introduzida no formato dd/mm/aaaa ou dd/mm/aa.
        >>> Se o ano tiver apenas dois digitos, sera'
        >>> assumido o seculo atual.

        >>> Digite a data no formato dd/mm/aaaa ou dd/mm/aa:
        > 21/12/10

        >>> A data introduzida foi:
        > 21/12/2010

 Exemplo 2:
 =========

        >>> Este programa verifica a validade de uma data
        >>> introduzida no formato dd/mm/aaaa ou dd/mm/aa.
        >>> Se o ano tiver apenas dois digitos, sera'
        >>> assumido o seculo atual.

        >>> Digite a data no formato dd/mm/aaaa ou dd/mm/aa:
        > 21/12/199d

        >>> Data invalida. Tente novamente <<<
        > 21/12/1999

        >>> A data introduzida foi:
        > 21/12/1999

***********************************************************/
