/****
 *
 * Ttulo: Comparando Strings sem Diferenciar Maisculas e Minsculas
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 11/07/2012
 * ltima modificao: 14/04/2013
 *
 * Descrio: Compara dois strings sem levar em considerao
 *            diferenas entre maisculas e minsculas
 *
 * Entrada: No h
 *
 * Sada: Resultados de comparaes de strings
 *
 ****/

#include <stdio.h>  /* Funo printf()  */
#include <ctype.h>  /* Funo toupper() */
#include <string.h> /* Funo strcmp()  */

/****
 *
 * ComparaStr(): Compara dois strings sem levar em considerao
 *               diferenas entre letras maisculas e minsculas
 *
 * Parmetros: str1 (entrada) - primeiro string a comparar
 *             str2 (entrada) - segundo string a comparar
 *
 * Retorno: 0, se str1 = str2
 *          < 0 se str1 < str2
 *          > 0, se str1 > str2
 *
 ****/
int ComparaStr(const char *str1, const char *str2)
{
   /************************************************************/
   /* Enquanto nenhum dos ponteiros apontar para '\0', compara */
   /* respectivos caracteres convertidos em maisculos. Se     */
   /* forem encontrados dois caracteres diferentes, retorna a  */
   /* diferena entre os valores do primeiro e do segundo      */
   /* caracteres convertidos em maisculos.                    */
   /************************************************************/

      /* O lao encerra quando 'str1' ou 'str2' apontar para */
      /* '\0' ou quando for encontrada uma diferena entre   */
      /* respectivos caracteres convertidos em maisculos    */
   while (*str1 && *str2) {
         /* Se os caracteres maisculos sendo comparados */
         /* diferem, retorna a diferena entre os        */
         /* respectivos caracteres maisculos            */
      if (toupper(*str1) != toupper(*str2)) {
            /* O uso de toupper()  necessrio para produzir */
            /* a ordem desejada. Por exemplo, suponha que os */
            /* caracteres so 'a' e 'Z'. Ento, se toupper() */
            /* no fosse usada, a diferena entre 'a' e 'Z'  */
            /* seria positiva, de modo que o string contendo */
            /* 'a' seria considerado sucessor daquele que    */
            /* contm 'Z'.                                   */
         return toupper(*str1) - toupper(*str2);
      }

         /* Faz cada ponteiro apontar para o  */
         /* prximo caractere a ser comparado */
      ++str1;
      ++str2;
   }

      /*                                                  */
      /* Se 'str1' e 'str2' apontam para seus respectivos */
      /* caracteres terminais, os strings so iguais (j  */
      /* que eles tm o mesmo tamanho). Caso contrrio,   */
      /* o string de maior tamanho ser o maior.          */
      /*                                                  */
   if (!*str1 && !*str2) { /* Strings tm o mesmo tamanho */
      return 0; /* Os strings so considerados iguais */
   }

      /* Neste ponto, sabe-se que um dos ponteiros aponta */
      /* para '\0', mas o outro no aponta para '\0'. Se  */
      /* o caractere apontado por 'str1' no for '\0',    */
      /* esse ponteiro apontava para o maior string. Caso */
      /* contrrio, ele apontava para o menor string. A   */
      /* instruo return a seguir lida com os dois casos */
   return *str1 - *str2;
}

/****
 *
 * main(): Testa a funo ComparaStr() e compara seus resultados
 *         com aqueles apresentados por strcmp()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char s1[] = "Zebra",
        s2[] = "abelha",
        s3[] = "ZEBRA";
   int teste;

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa compara strings sem levar em"
           "\n\t>>> consideracao diferencas entre maiusculas e"
           "\n\t>>> minusculas, e compara os resultados com"
           "\n\t>>> aqueles apresentados por strcmp().\n" );

      /*                                     */
      /* Compara s1[] e s2[] usando strcmp() */
      /*                                     */

   printf("\n\t>>> Usando strcmp() <<<\n");
   teste = strcmp(s1, s2);

   if (teste > 0) {
      printf("\n\t>>> \"%s\" sucede \"%s\"\n", s1, s2);
   } else if (teste < 0){
      printf("\n\t>>> \"%s\" antecede \"%s\"\n", s1, s2);
   } else {
      printf("\n\t>>> \"%s\" e \"%s\" sao iguais\n", s1, s2);
   }

      /*                                         */
      /* Compara s1[] e s2[] usando ComparaStr() */
      /*                                         */

   printf("\n\n\t>>> Usando ComparaStr() <<<\n");
   teste = ComparaStr(s1, s2);

   if (teste > 0) {
      printf("\n\t>>> \"%s\" sucede \"%s\"\n", s1, s2);
   } else if (teste < 0){
      printf("\n\t>>> \"%s\" antecede \"%s\"\n",
              s1, s2);
   } else {
      printf("\n\t>>> \"%s\" e \"%s\" sao iguais\n", s1, s2);
   }

      /*                                     */
      /* Compara s1[] e s3[] usando strcmp() */
      /*                                     */

   printf("\n\n\t>>> Usando strcmp() <<<\n");
   teste = strcmp(s1, s3);

   if (teste > 0) {
      printf("\n\t>>> \"%s\" sucede \"%s\"\n", s1, s3);
   } else if (teste < 0){
      printf("\n\t>>> \"%s\" antecede \"%s\"\n", s1, s3);
   } else {
      printf("\n\t>>> \"%s\" e \"%s\" sao iguais\n", s1, s3);
   }

      /*                                         */
      /* Compara s1[] e s3[] usando ComparaStr() */
      /*                                         */

   printf("\n\n\t>>> Usando ComparaStr() <<<\n");
   teste = ComparaStr(s1, s3);

   if (teste > 0) {
      printf("\n\t>>> \"%s\" sucede \"%s\"\n", s1, s3);
   } else if (teste < 0){
      printf("\n\t>>> \"%s\" antecede \"%s\"\n", s1, s3);
   } else {
      printf("\n\t>>> \"%s\" e \"%s\" sao iguais\n", s1, s3);
   }

   return 0;
}

/****************** Resultado do Programa ******************

        >>> Este programa compara strings sem
        >>> levar em consideracao diferencas
        >>> entre maiusculas e minusculas, e
        >>> compara os resultados com aqueles
        >>> apresentados por strcmp().

        >>> Usando strcmp() <<<

        >>> "Zebra" antecede "abelha"


        >>> Usando ComparaStr() <<<

        >>> "Zebra" sucede "abelha"


        >>> Usando strcmp() <<<

        >>> "Zebra" sucede "ZEBRA"


        >>> Usando ComparaStr() <<<

        >>> "Zebra" e "ZEBRA" sao iguais

***********************************************************/
