/****
 *
 * Ttulo: Processando Dados de um Aluno
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/05/2012
 * ltima modificao: 16/05/2012
 *
 * Descrio Geral: L via teclado e exibe na tela dados
 *                  dados de uma estrutura do tipo tAluno
 *
 * Entrada: v. Exemplo de Execuo
 *
 * Sada: v. Exemplo de Execuo
 *
 ****/

/************************ Includes ************************/

#include <stdio.h>   /* Entrada e sada             */
#include <string.h>  /* Processamento de strings    */
#include <ctype.h>   /* Classificao de caracteres */
#include "leitura.h" /* LeituraFacil                */

/****************** Constantes Simblicas *****************/

#define MAX_NOME 20 /* Nmero mximo de caracteres em nome */
#define TAM_MATR  4 /* Nmero de dgitos numa matrcula */

/******************* Definies de Tipos ******************/

   /* Tipo dos dados de um aluno */
typedef struct {
           char   nome[MAX_NOME + 1]; /* Nome do aluno */
           char   matr[TAM_MATR + 1]; /* Matrcula     */
           double n1, n2;             /* Notas 1 e 2   */
        } tAluno;

/************************* Aluses ************************/

extern int LeNome(char *nome, int tam);
extern char *LeIdentidade( char *id, int tamArray,
                           const char *prompt );
extern double LeNota(void);
extern tAluno *LeDadosAluno(tAluno *aluno, int fMatr);
extern void ExibeDadosAluno( const tAluno *dados );

/****************** Definies de Funes *****************/

/****
 *
 * LeNome(): L um string via teclado e garante
 *           que ele s contm letras e espaos
 *           em branco
 *
 * Parmetros: nome (sada) - array que conter
 *                            o string lido
 *             tam (entrada) - nmero de elementos
 *                             do array
 *
 * Retorno: O nmero de caracteres digitados alm do limite
 *          permitido
 *
 * Observaes:
 *      1. Caracteres digitados alm do limite especificado
 *         sero ignorados
 *
 *      2. O nico espao em branco permitido num nome  ' '
 *
 *      3. Esta funo no testa se o string digitado 
 *         constitudo apenas por espaos em branco
 *
 ****/
int LeNome(char *nome, int tam)
{
   int   remanescentes; /* Nmero de caracteres digitados */
                        /* alm do permitido              */
   char *p;

   while (1) {
      printf( "\nDigite o nome (max = %d letras):\n\t> ",
              tam - 1 );
      remanescentes = LeString(nome, tam);

         /* Verifica se o string lido  vazio */
      if (!*nome) {
         printf("\a\n\t>> Um nome nao pode ser vazio <<\n");
         continue; /* Salta o restante do lao */
      }

         /***************************************/
         /*                                     */
         /* Verifica se o string lido s contm */
         /* letras e espaos em branco          */
         /*                                     */
         /***************************************/

         /* O ponteiro p ser usado para testar o string */
      p = nome;

         /* O lao encerra quando p apontar para o */
         /* caractere terminal do string ou quando */
         /* for encontrado um caractere que nem   */
         /* letra nem espao em branco             */
      while (*p) {
            /* Se for encontrado um caractere que */
            /* no  letra ou espao em branco    */
            /* encerra este lao                  */
         if( !isalpha(*p) && *p != ' ' ) {
            break;
         }

         p++; /* Aponta para o prximo caractere */
      }

         /* Se p apontar para o caractere terminal do */
         /* string, este foi aprovado no teste acima  */
      if (!*p) {
         break; /* String s contm letras e espaos */
      } else { /* String foi reprovado no teste */
         printf("\a\n\t>>> Um nome deve conter apenas <<<"
                "\n\t>>> letras e espacos em branco <<<\n");
      }
   }

   return remanescentes;
}

/****
 *
 * LeIdentidade(): L um nmero de identificao via teclado
 *
 * Parmetros: id[] (sada) - array que conter o nmero de
 *                            identificao lido
 *             tamArray (entrada) - nmero de elementos
 *                                  do array id[]
 *             prompt (entrada) - string que representa
 *                                a poro inicial do
 *                                prompt a ser apresentado
 *
 * Retorno: O endereo do array apontado por id
 *
 * Observao: Assume-se que o tamanho do nmero de
 *             identificao  igual ao tamanho do array
 *             id[] que a armazenar menos um.
 *
 ****/
char *LeIdentidade( char *id, int tamArray,
                    const char *prompt )
{
   int   resto; /* Caracteres excedentes que */
                /* possam ter sido digitados */
   char *p;

      /* O lao while encerra quando */
      /* for lido um string vlido   */
   while (1) {
         /* Apresenta prompt */
      printf( "\n%s com exatamente %d digitos:\n\t> ",
              prompt, tamArray - 1 );

         /* L entrada do usurio */
      resto = LeString(id, tamArray);

         /* Verifica se o usurio digitou */
         /* caracteres em excesso         */
      if (resto) {
         printf("\a\n\t>>> Excesso de caracteres <<<\n");
         continue; /* Faz nova tentativa */
      }

         /* Verifica se foram digitados menos caracteres */
         /* do que esperado. Note o uso do operador de   */
         /* converso explcita (int) para converter o   */
         /* valor retornado por strlen(), que  do tipo  */
         /* size_t, e, assim, evitar a comparao de um  */
         /* valor sem sinal e outro com sinal.           */
      if ( (int)strlen(id) != tamArray - 1 ) {
         printf("\a\n\t>>> Numero incompleto <<<\n");
         continue; /* Faz nova tentativa */
      }

         /*************************************/
         /*                                   */
         /* Neste ponto, o tamanho do nmero  */
         /* est correto. Resta testar se ele */
         /* s contm dgitos.                */
         /*                                   */
         /*************************************/

         /* O ponteiro p ser usado para testar o string */
      p = id;

         /* O lao encerra quando p apontar para o     */
         /* caractere terminal do string ou quando for */
         /* encontrado um caractere que no  dgito   */
      while (*p) {
            /* Se for encontrado um caractere que */
            /* no  dgito encerra este lao     */
         if(!isdigit(*p)) {
            break; /* Encerra o lao interno */
         }

         p++; /* Aponta para o prximo caractere */
      }

         /* Se p apontar para o caractere terminal do */
         /* string, este foi aprovado no teste acima  */
      if (!*p) {
         break; /* Nmero de identificao vlido */
      } else { /* Nmero de identificao invlido */
         printf("\a\n\t>>> \'%c\' nao e' digito <<<\n", *p);
      }
   }

   return id;
}

/****
 *
 * LeNota(): L um valor x do tipo double tal que 0.0 <= x <= 10.0
 *
 * Parmetros: Nenhum
 *
 * Retorno: O valor lido
 *
 ****/
double LeNota(void)
{
   double valorLido;

      /* O lao while encerra apenas quando */
      /* for lido um valor vlido           */
   while (1) {

      valorLido = LeReal();

         /* Verifica se o valor lido est */
         /* de acordo com o esperado      */
      if (valorLido >= 0.0 && valorLido <= 10.0) {
         break; /* O valor lido est OK */
      } else {
         printf( "\n\t>>> Nota incorreta. "
                 "Tente novamente\n\t> " );
      }
   }

   return valorLido;
}

/****
 *
 * LeDadosAluno(): L os dados de um aluno
 *
 * Parmetros:
 *     aluno (sada) - ponteiro para a estrutura que receber os
 *                     dados introduzidos pelo usurio
 *     fMatr (entrada) - indica se o campo 'matr' ser lido ou no
 *
 * Retorno: Endereo da estrutura que armazena os dados lidos
 *
 * Observao: O parmetro 'fMatr'  uma flag que determina se a
 *             matrcula deve ser lida ('fMatr' diferente de zero)
 *             ou no ('fMatr' igual a zero). A justificativa para
 *             o uso desse parmetro  que, algumas vezes, quando
 *             esta funo  chamada, a matrcula j ter sido
 *             lida. Essa alternativa  melhor do que solicitar
 *             novamente a matrcula ao usurio (inconveniente)
 *             ou escrever duas funes que diferem apenas pelo
 *             fato de solicitarem matrcula ou no.
 *
 ****/
tAluno *LeDadosAluno(tAluno *aluno, int fMatr)
{
   LeNome(aluno->nome, MAX_NOME + 1); /* L o nome */

      /* Se indicado pelo parmetro 'fMatr', l a matrcula */
   if (fMatr) {
      LeIdentidade(aluno->matr, TAM_MATR+1, "Digite a matricula");
   }

      /* L a primeira nota */
   printf("\n\t>>> 1a. nota:\n\t    > ");
   aluno->n1 = LeNota();

      /* L a segunda nota */
   printf("\n\t>>> 2a. nota:\n\t    > ");
   aluno->n2 = LeNota();

   return aluno;
}

/****
 *
 * ExibeDadosAluno(): Exibe na tela os campos de uma estrutura do
 *                    tipo tAluno
 *
 * Parmetros:
 *     dados (entrada) - ponteiro para a estrutura que ser
 *                       apresentada na tela
 *
 * Retorno: Nada
 *
 ****/
void ExibeDadosAluno( const tAluno *dados )
{
   printf("\n>>> Nome:\t%s", dados->nome);
   printf("\n>>> Matricula:\t%s", dados->matr);
   printf("\n>>> Nota 1:\t%3.1f", dados->n1);
   printf("\n>>> Nota 2:\t%3.1f\n", dados->n2);
}

/****
 *
 * main(): Testa as funes LeDadosAluno() e ExibeDadosAluno()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   tAluno umAluno;

   printf( "\n\t>>> Este programa le e apresenta dados de "
           "um aluno\n" );

      /* L dados de um aluno, incluindo sua matrcula */
   LeDadosAluno(&umAluno, 1);

      /* Apresenta os dados do aluno */
   printf("\n\t>>> Dados do aluno <<<\n");
   ExibeDadosAluno(&umAluno);

   return 0;
}

/******************* Exemplo de Execuo *******************

        >>> Este programa le e apresenta dados de um aluno

Digite o nome (max = 20 letras):
        > Henrique VIII

Digite a matricula com exatamente 4 digitos:
        > 1029

        >>> 1a. nota:
            > 9.5

        >>> 2a. nota:
            > 9.0

        >>> Dados do aluno <<<

>>> Nome:       Henrique VIII
>>> Matricula:  1029
>>> Nota 1:     9.5
>>> Nota 2:     9.0

***********************************************************/
