/****
 *
 * Ttulo: Alinhamento de Strings
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Alinha um string  esquerda,  direita ou ao centro de
 *            um array e preenche o espao restante com um caractere
 *
 * Entrada: No h
 *
 * Sada: O string original e ele alinhado  esquerda,
 *         direita ou ao centro
 *
 ****/

#include <string.h> /* strlen() */
#include <stdio.h>  /* printf() */

   /* Tamanho do array que armazena os strings */
#define TAM_ARRAY 30

   /* Tipos de alinhamento */
typedef enum {ESQUERDA, CENTRO, DIREITA} tAlinhamento;

/****
 *
 * AlinhaString(): alinha um string  esquerda,  direita ou ao
 *                 centro de um array, preenchendo o espao
 *                 restante com um caractere especificado
 *
 * Parmetros:
 *      ar[] (sada) - array que conter o string alinhado
 *      largura (entrada) - tamanho do array ar[]
 *      preenchimento (entrada) - caractere usado no preenchimento
 *                                do espao restante
 *      alinhamento (entrada) - o tipo de alinhamento
 *      str (entrada) - o string que ser alinhado
 *
 * Retorno: Endereo do array contendo o resultado da operao
 *
 * Nota: Se o comprimento do string str for maior do que o
 *       tamanho do array, ele ser truncado.
 *
 ****/
char *AlinhaString( char ar[], int largura, char preenchimento,
                    tAlinhamento alinhamento, const char *str )
{
   char *p;
   int   i,
         menorLargura,/* O menor valor entre o tamanho do */
                      /* array e o comprimento do string  */
         tamStr; /* Comprimento do string */

      /* Calcula o tamanho do string */
   tamStr = strlen(str);

      /* Determina a menor largura */
   menorLargura = largura < tamStr ? largura : tamStr;

      /* Preenche todo o array com o caractere de preenchimento */
   for (i = 0; i < largura - 1; ++i) {
      ar[i] = preenchimento;
   }

      /* Transforma o contedo do array num string */
   ar[largura - 1] = '\0';

      /* Determina a posio inicial do string dentro do array */
   if (alinhamento == ESQUERDA) {
      p = ar;
   } else if (alinhamento == CENTRO) {
      p = ar + (largura - menorLargura)/2;
   } else { /* alinhamento == DIREITA */
      p = ar + largura - menorLargura - 1;
   }

         /*                                                */
         /* Neste ponto, p aponta para o elemento do array */
         /* que receber o primeiro caractere do string    */
         /*                                                */

      /* Insere o string em sua posio */
   for (i = 0; i < menorLargura; ++i) {
      p[i] = str[i];
   }

      /* Retorna o endereo do array  */
      /* que contm o string alinhado */
   return ar;
}

/****
 *
 * main(): Testa a funo AlinhaString()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char  ar[TAM_ARRAY];
   char *str = "Um string";

      /* Apresenta o string original */
   printf("\nString original: \"%s\"\n", str);

         /*                                    */
         /* Apresenta o string alinhado       */
         /* esquerda,  direita e centralizado */
         /*                                    */

   AlinhaString(ar, TAM_ARRAY, '*', ESQUERDA, str);
   printf( "\nString alinhado a esquerda:\n\t> \"%s\"\n",
           ar );

   AlinhaString(ar, TAM_ARRAY, '*', DIREITA, str);
   printf( "\nString alinhado a direita:\n\t> \"%s\"\n",
           ar );

   AlinhaString(ar, TAM_ARRAY, '*', CENTRO, str);
   printf("\nString centralizado:\n\t> \"%s\"\n", ar);

   return 0;
}

/****************** Resultado do Programa ******************

String original: "Um string"

String alinhado a esquerda:
        > "Um string********************"

String alinhado a direita:
        > "********************Um string"

String centralizado:
        > "**********Um string**********"

***********************************************************/
