/****
 *
 * Ttulo: Distncias entre Cidades
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Apresenta a distncia de uma cidade at o Rio de Janeiro
 *
 * Entrada: Nome da cidade que o usurio deseja pesquisar
 *
 * Sada: A referida distncia se o nome de cidade introduzido pelo
 *        usurio for encontrado.
 *
 ****/

/************************ Includes ************************/

#include <stdio.h>   /* printf() e putchar() */
#include <string.h>  /* strcmp()             */
#include "leitura.h" /* LeString()           */

/****************** Constantes Simblicas *****************/

   /* Tamanho do array que armazena os strings lidos */
#define TAMANHO_ARRAY 50

/******************* Definies de Tipos ******************/

typedef struct {
           char *cidade;
           int   distancia;
        } tCidadeDistancia;

/****************** Definies de Funes *****************/

/****
 *
 * ApresentaCidades(): Apresenta na tela o campo 'cidade' de cada
 *                     estrutura do tipo tCidadeDistancia de um
 *                     array de elementos desse tipo
 *
 * Parmetros:
 *      ar[] (entrada) - array que contm as estruturas
 *      tam (entrada) - nmero de elementos do array
 *
 * Retorno: Nada
 *
 ****/
void ApresentaCidades(const tCidadeDistancia ar[], int tam)
{
   int i;

   printf("\n\n\t        >>> Cidades Disponiveis <<<\n\n");

      /* Exibe na tela quatro cidades por linha */
   for (i = 0; i < tam; ++i) {
      printf("%-15s\t", ar[i].cidade);

         /* Quebra linha a cada 4 cidades escritas */
      if (!((i + 1)%4)) {
         putchar('\n');
      }
   }
}

/****
 *
 * main(): Apresenta a distncia de uma cidade at o Rio de Janeiro
 *         (se essa informao estiver disponvel, obviamente)
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char                   cidade[TAMANHO_ARRAY];
   int                    i, nCidades;
   const tCidadeDistancia distancias[] =
     {
       {"Amsterdam",      9795}, {"Bagdad",        11168},
       {"Berlim",        10004}, {"Beirute",       10429},
       {"Jerusalem",     10306}, {"Joao Pessoa",    2448},
       {"Lisboa",         7711}, {"Londres",        9486},
       {"Moscou",        11542}, {"Nova Deli",     14068},
       {"Nova Iorque",    7763}, {"Oslo",          10415},
       {"Otawa",          8283}, {"Paris",          9162},
       {"Viena",          9871}, {"Zurich",         9610}
     };

   printf("\n\t>>> Este programa apresenta a distancia entre o "
          "Rio\n\t>>> de Janeiro e uma cidade escolhida pelo "
          "usuario.\n\t>>> Digite apenas [ENTER] para encerrar.");

      /* Calcula o nmero de elementos do array */
   nCidades = sizeof(distancias)/sizeof(distancias[0]);

      /* O lao encerra quando o usurio */
      /* digitar apenas [ENTER]          */
   while (1) {
         /* Apresenta ao usurio as cidades disponveis */
      ApresentaCidades(distancias, nCidades);

         /* L o nome da cidade */
      printf("\n>>> Escolha uma cidade para conhecer sua"
             " distancia ate' o\n>>> Rio de Janeiro > ");
      LeString(cidade, TAMANHO_ARRAY);

      if (!*cidade) {
         break; /* Usurio digitou apenas [ENTER] */
      }

         /* Procura a cidade */
      for (i = 0; i < nCidades; ++i) {
         if (!strcmp(cidade, distancias[i].cidade)) {
            break; /* A cidade foi encontrada */
         }
      }

         /* Se o valor de i na sada do lao for igual */
         /* a 'nCidades', a cidade no foi encontrada  */
      if (i == nCidades) {
         printf("\n>>> A cidade \"%s\" nao foi encontrada",
                 cidade);
      } else {
         printf( "\n>>> A distancia do Rio de Janeiro ate' %s "
                 "e' %dKm", cidade, distancias[i].distancia );
      }
   } /* while */

   printf( "\n>>> Obrigado por usar este programa.\n");

  return 0;
}

/******************* Exemplo de Execuo *******************

        >>> Este programa apresenta a distancia entre o Rio
        >>> de Janeiro e uma cidade escolhida pelo usuario.
        >>> Digite apenas [ENTER] para encerrar.

                >>> Cidades Disponiveis <<<

Amsterdam       Bagdad          Berlim          Beirute
Jerusalem       Joao Pessoa     Lisboa          Londres
Moscou          Nova Deli       Nova Iorque     Oslo
Otawa           Paris           Viena           Zurich

>>> Escolha uma cidade para conhecer sua distancia ate' o
>>> Rio de Janeiro > Campina Grande

>>> A cidade "Campina Grande" nao foi encontrada

                >>> Cidades Disponiveis <<<

Amsterdam       Bagdad          Berlim          Beirute
Jerusalem       Joao Pessoa     Lisboa          Londres
Moscou          Nova Deli       Nova Iorque     Oslo
Otawa           Paris           Viena           Zurich

>>> Escolha uma cidade para conhecer sua distancia ate' o
>>> Rio de Janeiro > Joao Pessoa

>>> A distancia do Rio de Janeiro ate' Joao Pessoa e' 2448Km

                >>> Cidades Disponiveis <<<

Amsterdam       Bagdad          Berlim          Beirute
Jerusalem       Joao Pessoa     Lisboa          Londres
Moscou          Nova Deli       Nova Iorque     Oslo
Otawa           Paris           Viena           Zurich

>>> Escolha uma cidade para conhecer sua distancia ate' o
>>> Rio de Janeiro >

>>> Obrigado por usar este programa.

***********************************************************/
