/****
 *
 * Exemplo: Fechando um Arquivo
 *
 * Seo: 11.5
 *
 ****/

#include <stdio.h>  /* Entrada e sada */
#include <stdlib.h> /* Funo exit()   */

   /* Nome do arquivo usado para testar o programa */
#define NOME_ARQ "Teste.txt"

/****
 *
 * FechaArquivo(): Tenta fechar um arquivo e, quando isso
 *                 no  possvel, aborta o programa
 *
 * Parmetros:
 *      stream (entrada) - stream associado ao arquivo
 *      nomeArq (entrada) - nome do arquivo ou NULL
 *
 * Retorno: Nada
 *
 * Observao: Se o segundo parmetro no for NULL, o nome
 *             do arquivo aparece na mensagem de erro
 *
 ****/
void FechaArquivo(FILE *stream, const char *nomeArq)
{
   /***********************************************************/
   /* Se fclose() retornar um valor diferente de zero ocorreu */
   /* algum erro na tentativa de fechamento do arquivo. Nesse */
   /* caso, apresenta mensagem de erro e aborta o programa.   */
   /***********************************************************/

   if (fclose(stream)) { /* Erro de fechamento */
      fprintf( stderr, "\a\n>>> Ocorreu erro no fechamento "
              "do arquivo %s.\n>>> O programa sera' "
              "encerrado.\n", nomeArq ? nomeArq : "");
      exit(1); /* Aborta o programa */
   }
}

/****
 *
 * main(): Testa a funo FechaArquivo()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(void)
{
   FILE *stream;
   char  str[] = "Este e' um arquivo de texto";

   stream = fopen(NOME_ARQ, "w");

      /* Testa se arquivo foi aberto */
   if (!stream) {
      fprintf(stderr, "\a\nArquivo nao pode ser aberto\n");
      return 1;
   }

      /* Escreve contedo do string armazenado */
      /* em str[] no arquivo                   */
   fprintf(stream, "%s", str);

      /* Fora a descarga do buffer associado ao arquivo */
   fflush(stream);

      /* Solicita que o usurio remova o arquivo sendo   */
      /* processado pelo programa. Mas, nem todo usurio */
      /* ser capaz de realizar essa tarefa porque o     */
      /* arquivo provavelmente estar bloqueado pelo SO. */
   printf( "\n\t>>> Remova o arquivo \"%s\" e digite uma"
           "\n\t>>> tecla depois de fazer isso", NOME_ARQ );
   getchar(); /* L um caractere seguido de [ENTER] */

      /* Tenta fechar o arquivo e, se no    */
      /* conseguir, o programa ser abortado */
   FechaArquivo(stream, NOME_ARQ);

      /* Esta instruo s ser executada se o   */
      /* usurio no conseguir remover o arquivo */
   return 0;
}

/******************* Exemplo de Execuo ******************

        >>> Remova o arquivo "Teste.txt" e digite uma
        >>> tecla depois de fazer isso

>>> Ocorreu erro no fechamento do arquivo Teste.txt.
>>> O programa sera' encerrado.

**********************************************************/
