/****
 *
 * Exemplo: A Funo getchar()
 *
 * Seo: 11.9.1
 *
 ****/

#include <stdio.h>
#include <ctype.h>

   /* Nmero mximo de caracteres permitidos, sem */
   /* considerar caracteres em branco iniciais    */
#define TAMANHO_MAX_LINHA 30

/****
 *
 * LeLinha(): L uma linha de texto introduzida pelo usurio no
 *            meio de entrada padro desprezando espaos em
 *            branco iniciais
 *
 * Parmetros:
 *     str[] (sada) - array no qual os caracteres lidos mais '\0'
 *                     sero armazenados
 *     n (entrada) - tamanho do array str[]
 *
 * Retorno: O nmero de espaos em branco iniciais saltados
 *
 ****/
int LeLinha(char str[], int n)
{
   int c, /* Armazena cada caractere lido */
       i = 0, /* Indexador de caracteres armazenados */
       espacosSaltados = 0; /* Nmero de espaos saltados */

      /* Salta espaos no incio da linha. O ltimo caractere  */
      /* atribudo a c no  espao em branco e encerra o lao */
   while ( isspace(c = getchar()) ) {
      ++espacosSaltados;
   }

      /* L os demais caracteres da linha */
   while (c != '\n' && c != EOF) {
         /* Verifica se o nmero mximo de */
         /* caracteres permitido foi lido  */
      if (i >= n - 1) { /* Limite atingido */
         break; /* Encerra o lao */
      }

      str[i] = c; /* Armazena o caractere lido */

      i++; /* Obtm o ndice do prximo caractere */

      c = getchar(); /* L o prximo caractere */
   }

   str[i] = '\0'; /* Termina o string */

   return espacosSaltados;
}

/****
 *
 * main(): Testa a funo LeLinha()
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   char str[TAMANHO_MAX_LINHA + 1];
   int  nEspacosSaltados;

   printf( "\n>>> Digite uma linha de texto com espacos em"
           "\n>>> branco no inicio (Maximo = %d caracteres)"
           ":\n\t> ", TAMANHO_MAX_LINHA );

   nEspacosSaltados = LeLinha(str, TAMANHO_MAX_LINHA);

   printf("\n>>> String sem espacos iniciais: \"%s\"", str);

   printf( "\n>>> Numero de espacos em branco saltados: %d\n",
           nEspacosSaltados );

   return 0;
}

/******************* Exemplo de Execuo ******************

>>> Digite uma linha de texto com espacos em
>>> branco no inicio (Maximo = 30 caracteres):
        >       Bom dia

>>> String sem espacos iniciais: "Bom dia"
>>> Numero de espacos em branco saltados: 6

**********************************************************/
