/****
 *
 * Ttulo: Filtro de Numerao de Linhas
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 16/06/2012
 * ltima modificao: 16/06/2012
 *
 * Descrio: Filtro que acrescenta numerao de linhas para
 *            arquivos de texto
 *
 * Entrada: Contedo de um arquivo de texto
 *
 * Sada: Contedo do arquivo com as linhas precedidas por seus
 *        nmeros de ordem
 *
 * Usos (supondo que o nome do executvel  'numera'):
 *
 *     1. type nome-de-arquivo-texto | numera | more (DOS)
 *                               ou
 *     1. cat nome-de-arquivo-texto | numera | more (Unix)
 *
 *     2. numera < nome-de-arquivo-texto
 *               > nome-de-arquivo-texto-numerado (Unix/DOS)
 *
 ****/

#include <stdio.h>

/****
 *
 * main(): Implementa um filtro que acrescenta numerao de
 *         linhas para arquivos de texto
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero
 *
 ****/
int main(void)
{
   int linha = 0, /* Contadora de linhas */
       c, /* Armazena cada caractere lido */
       novaLinha = 1; /* Indica se o prximo caractere  */
                      /* o incio de uma linha nova      */

      /* Enquanto o final do arquivo associado a */
      /* stdin no for atingido, l caracteres e */
      /* numera cada nova linha encontrada       */
   while ( (c = getchar()) != EOF ) {
         /* Uma linha nova  a primeira ou   */
         /* aquela que comea logo aps '\n' */
      if (novaLinha) { /* Incio de nova linha */
         printf("%.4d: ", ++linha); /* Escreve numerao */
         novaLinha = 0; /* Desliga indicao de linha nova */
      }

      putchar(c); /* Escreve o caractere lido em stdout */

         /* Se o ltimo caractere lido foi '\n', o  */
         /* prximo caractere inicia uma nova linha */
      if (c == '\n') {
         novaLinha = 1; /* Indica incio de nova linha */
      }
   }

   return 0;
}

/******************* Exemplo de Execuo ******************

C:\Programas>numera < AnedotaBulgara.txt
0001: Anedota Bulgara
0002: Carlos Drummond de Andrade
0003:
0004: Era uma vez um czar naturalista
0005: que cacava homens.
0006: Quando lhe disseram que tambem se
0007: cacam borboletas e andorinhas,
0008: ficou muito espantado
0009: e achou uma barbaridade

[O nome do programa executvel  numera]

**********************************************************/
