/****
 *
 * Ttulo: Nmero de Linhas de um Arquivo de Texto
 *
 * Autor: Ulysses de Oliveira
 *
 * Data de Criao: 19/05/2012
 * ltima modificao: 19/05/2012
 *
 * Descrio: Conta o nmero de linhas de um arquivo de texto
 *
 * Entrada: Nome do arquivo introduzido via linha de comando
 *
 * Sada: Uma mensagem informando o nmero de linhas do arquivo
 *
 ****/

#include <stdio.h> /* Entrada e sada */

/****
 *
 * NumeroDeLinhas(): Conta o nmero de linhas de um arquivo de texto
 *
 * Parmetros:
 *      stream (entrada): stream associado ao arquivo
 *
 * Retorno: O nmero de linhas do arquivo ou  um valor negativo
 *          se ocorrer erro
 *
 * Observao: O arquivo deve ter sido aberto em modo de texto que
 *             permite leitura
 *
 ****/
int NumeroDeLinhas(FILE *stream)
{
   int    c, /* Armazena temporariamente um caractere */
          linhas = 0; /* Armazena o total de linhas */

      /* Garante leitura a partir do incio do arquivo */
   rewind(stream);

      /* L cada caractere do stream e conta quantos caracteres */
      /* '\n' so encontrados. O lao while encerra quando o    */
      /* final do arquivo  atingido ou ocorre erro de leitura. */
   while (1) {
      c = fgetc(stream); /* L um caractere no stream */

         /* Se o final do arquivo foi atingido    */
         /* ou ocorreu algum erro, encerra o lao */
      if ( feof(stream) || ferror(stream) )
         break;

      if (c == '\n') {
         linhas++; /* Mais uma linha encontrada */
      }
   }

      /* Se ocorreu erro retorna -1. Caso      */
      /* contrrio, retorna o nmero de linhas */
   return ferror(stream) ? -1 : linhas;
}

/****
 *
 * main(): Conta o nmero de linhas de um arquivo de texto
 *
 * Parmetros:
 *     argc (entrada) - Nmero de argumentos de linha de comando
 *     argv (entrada) - Array de strings presentes na linha de
 *                      comando quando o programa  executado
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(int argc, char* argv[])
{
   int   nLinhas;
   FILE *stream;

      /* Verifica se o programa foi invocado corretamente */
   if (argc != 2) { /* No foi */
         /* Faltou o usurio informar o nome do arquivo */
      printf( "\n\t>>> Este programa deve ser usado assim:"
              "\n\t\t%s nome-do-arquivo\n", argv[0] );
      return 1;
   }

      /* Tenta abrir o arquivo para leitura em modo texto */
   stream = fopen(argv[1], "r");

      /* Se o arquivo no foi aberto, */
      /* no  possvel continuar     */
   if (!stream) {
      printf( "\nNao foi possivel abrir o arquivo %s\n",
              argv[1] );
      return 1;
   }

      /* Determina o nmero de linhas do arquivo */
   nLinhas = NumeroDeLinhas(stream);

      /* Verifica se ocorreu erro durante a operao. Se */
      /* for o caso, informa o usurio e aborta programa */
   if (nLinhas < 0) {
      printf( "\n\t>>> Ocorreu erro ao tentar calcular o numero "
              "\n\t>>> de linhas do arquivo \"%s\"\n", argv[1] );
      fclose(stream); /* Fecha arquivo antes de partir */
      return 1;
   }

      /* Apresenta o nmero e linhas do arquivo */
   printf( "\n\t>>> Numero de linhas do arquivo \"%s\": %d\n",
           argv[1], nLinhas );

   fclose(stream); /* Fecha arquivo */

   return 0;
}

/******************* Exemplo de Execuo *******************

[O nome do programa executvel  linhas]

C:\Programas>linhas Tudor.txt

        >>> Numero de linhas do arquivo "Tudor.txt": 7

***********************************************************/
