/****
 *
 * Ttulo: Exibindo Registros de um Arquivo Binrio na Tela
 *
 * Data de Criao: 17/05/2012
 * ltima modificao: 18/05/2012
 *
 * Descrio: L o contedo do arquivo binrio especificado e
 *            apresenta-o na tela
 *
 * Entrada: Contedo do arquivo especificado pela constante
 *          NOME_ARQUIVO_BIN
 *
 * Sada: Contedo do arquivo na tela
 *
 ****/

/******************* Arquivo de Entrada ********************

O arquivo processado por este programa est em FORMATO
BINRIO. Este arquivo contm as seguintes informaes:

Henrique VIII   1029   9.5   9.0
Catarina Aragon 1014   5.5   6.5
Ana Bolena      1012   7.8   8.0
Joana Seymour   1017   7.7   8.7
Ana de Cleves   1022   4.5   6.0
Catarina Howard 1340   6.0   7.7
Catarina Parr   1440   4.0   6.0

Mas, como este arquivo  binrio, se voc abri-lo num
editor de texto no visualizar as informaes conforme
aqui apresentadas.

************************ Includes *************************/

#include <stdio.h>   /* Entrada e sada */
#include <stdlib.h>  /* Funo exit()   */

/***************** Constantes Simblicas ******************/

#define MAX_NOME  20 /* Nmero mximo de caracteres num nome */
#define TAM_MATR   4 /* Nmero de dgitos numa matrcula */
#define NOME_ARQUIVO_BIN "Tudor.bin" /* Nome do arquivo binrio */

/****************** Definies de Tipos *******************/

typedef struct {
           char    nome[MAX_NOME + 1];
           char    matr[TAM_MATR + 1];
           double  n1, n2;
         } tAluno;

/***************** Definies de Funes ******************/

/****
 *
 * ExibeArquivoBin(): Exibe na tela os registros de um arquivo
 *                    binrio contendo estruturas do tipo tAluno
 *                    acessando-os sequencialmente
 *
 * Parmetros: stream (entrada) - stream associado ao arquivo
 *                                que ser exibido
 *
 * Retorno: Zero, se no houver erro.
 *          Um valor diferente de zero, em caso contrrio.
 *
 * Observao: O arquivo associado ao stream recebido como
 *             parmetro deve ter sido aberto em modo binrio
 *             que permite leitura
 *
 ****/
int ExibeArquivoBin(FILE *stream)
{
   tAluno  registro; /* Armazena cada registro lido */

      /* Move indicador de posio para o incio do arquivo.   */
      /* Se ocorrer erro, retorna um valor indicativo do fato. */
      /* A instruo a seguir  mais segura do que rewind().   */
   if (fseek(stream, 0, SEEK_SET)) {
      return 1; /* Indicador de posio no foi movido */
   }

   /*                                                     */
   /* Antes de apresentar o cabealho, deve-se tentar ler */
   /* o primeiro registro, pois se isso no for feito e o */
   /* arquivo estiver vazio ou ocorrer erro de leitura,   */
   /* o cabealho aparecer sem dados abaixo dele.        */
   /*                                                     */

      /* Tenta ler o primeiro registro */
   fread(&registro, sizeof(tAluno), 1, stream);

      /* Se o final do arquivo foi atingido ou */
      /* ocorreu erro de leitura, retorna 1    */
   if (feof(stream) || ferror(stream)) {
      return 1; /* Nada foi apresentado */
   }

      /* Escreve o cabealho da tabela */
   printf( "\n\tNome\t\tMatricula\tNota 1\tNota 2"
           "\n\t====\t\t=========\t======\t======\n\n" );

      /* Escreve o primeiro registro lido */
   printf( "    %-15s\t%6s\t\t %3.1f\t %3.1f\n",
           registro.nome, registro.matr, registro.n1,
           registro.n2 );

      /* Acessa cada registro do arquivo na ordem em   */
      /* que ele se encontra. O lao encerra quando o  */
      /* final do arquivo for atingido ou ocorrer erro */
   while (1) {
         /* L um registro */
      fread(&registro, sizeof(tAluno), 1, stream);

         /* Se o final do arquivo foi atingido ou   */
         /* ocorreu erro de leitura, encerra o lao */
      if (feof(stream) || ferror(stream)) {
         break;
      }

         /* Escreve o registro lido. No tente entender  */
         /* o embelezamento usado com printf(). Ele foi  */
         /* obtido por tentativa e erro, e no se aplica */
         /* a dados com outros formatos.                 */
      printf( "    %-15s\t%6s\t\t %3.1f\t %3.1f\n",
              registro.nome, registro.matr, registro.n1,
              registro.n2 );
   }

      /* Se no ocorreu erro de leitura, o retorno  0; */
      /* caso contrrio, o retorno  diferente de 0     */
   return ferror(stream);
}

/****
 *
 * main(): L o contedo do arquivo binrio especificado pela
 *         constante NOME_ARQUIVO_BIN e apresenta-o na tela
 *
 * Parmetros: Nenhum
 *
 * Retorno: Zero, se no ocorrer nenhum erro.
 *          Um valor diferente de zero em caso contrrio.
 *
 ****/
int main(void)
{
   FILE   *stream; /* Stream associado ao arquivo */

      /* Apresenta o programa */
   printf( "\n\t>>> Este programa exibe o conteudo do"
           "\n\t>>> arquivo binario \"%s\" na tela.\n",
           NOME_ARQUIVO_BIN );

      /* Tenta abrir o arquivo em modo */
      /* binrio para leitura apenas   */
   stream = fopen(NOME_ARQUIVO_BIN, "rb");

      /* Se o arquivo no foi aberto, no */
      /* h mais nada que se possa fazer  */
   if (!stream) {
      printf( "\n\t>>> O arquivo %s nao pode ser aberto\n",
              NOME_ARQUIVO_BIN );
      return 1; /* Arquivo no foi aberto */
   }

      /* Apresenta o contedo do arquivo na */
      /* tela checando ocorrncia de erro   */
   if (ExibeArquivoBin(stream)) {
      printf( "\n\t>>> Ocorreu erro de processamento do"
             "\n\t>>> arquivo \"%s\"\n", NOME_ARQUIVO_BIN );

      fclose(stream); /* Fecha arquivo antes de encerrar */

      return 1; /* Erro de processamento */
   }

   fclose(stream); /* Fecha o arquivo */

   return 0; /* Tudo ocorreu bem */
}

/******************* Exemplo de Execuo ******************

        >>> Este programa exibe o conteudo do
        >>> arquivo binario "Tudor.bin" na tela.

        Nome            Matricula       Nota 1  Nota 2
        ====            =========       ======  ======

    Henrique VIII         1029           9.5     9.0
    Catarina Aragon       1014           5.5     6.5
    Ana Bolena            1012           7.8     8.0
    Joana Seymour         1017           7.7     8.7
    Ana de Cleves         1022           4.5     6.0
    Catarina Howard       1340           6.0     7.7
    Catarina Parr         1440           4.0     6.0

**********************************************************/
