/****
 *
 * Exemplo: calloc()
 *
 * Seo: 11.2.2
 *
 ****/

#include <stdio.h>
#include <stdlib.h>

/****
 *
 * MinhaCalloc(): Aloca um nmero determinado de blocos de
 *                tamanho especificado e zera seus contedos
 *
 * Parmetros:
 *      nBlocos (entrada): nmero de blocos que sero alocados
 *      tamanho (entrada): tamanho de cada bloco
 *
 * Retorno: Endereo do primeiro byte alocado ou NULL se no
 *          houver alocao
 *
 ****/
void *MinhaCalloc(size_t nBlocos, size_t tamanho)
{
   size_t  i, nBytes;
   char   *ptr;

      /* Calcula o nmero de bytes que sero alocados */
   nBytes = nBlocos*tamanho;

   ptr = malloc(nBytes); /* Tenta alocar o bloco requisitado */

   if (!ptr) { /* Checa se houve alocao */
      return NULL; /* No foi possvel alocar o bloco */
   }

      /* Aqui, o espao j foi alocado. */
      /* Resta apenas zerar os bytes.   */
   for (i = 0; i < nBytes; ++i) {
      ptr[i] = 0; /* Zera cada byte */
   }

   return ptr; /* Trabalho completo */
}

int main(void)
{
   double *p;

   p = MinhaCalloc(2, sizeof(double));

   if (p) {
      printf("\n>>>>>> Antes das Atribuicoes <<<<<<\n");
      printf("\np[0] = %3.2f \t p[1] = %3.2f\n", p[0], p[1]);

      p[0] = 3.14;
      p[1] = 2.54;

      printf("\n>>>>>> Depois das Atribuicoes <<<<<<\n");
      printf("\np[0] = %3.2f \t p[1] = %3.2f\n", p[0], p[1]);
   } else {
      printf("\nAlocacao nao foi possivel\n");
   }

   return 0;
}

/****************** Resultado do Programa *****************

>>>>>> Antes das Atribuicoes <<<<<<

p[0] = 0.00      p[1] = 0.00

>>>>>> Depois das Atribuicoes <<<<<<

p[0] = 3.14      p[1] = 2.54

**********************************************************/
